<?php



if (defined('PS_ADMIN_DIR') && file_exists(PS_ADMIN_DIR . '/../classes/AdminTab.php'))
    include_once(PS_ADMIN_DIR . '/../classes/AdminTab.php');

require_once(dirname(__FILE__) . '/amazon.php');

require_once(dirname(__FILE__) . '/classes/amazon.webservice.class.php');
require_once(dirname(__FILE__) . '/classes/amazon.product.class.php');
require_once(dirname(__FILE__) . '/classes/amazon.tools.class.php');

class ProductsAmazon extends AdminTab
{
    private $module = 'amazon';
    private $_amazon;

    public $multishop_context ;
    public $multishop_context_group ;
  
    public $id_lang ;
    
    public function __construct()
    {
        global $cookie, $_LANGADM, $_MODULE;

        if ( version_compare(_PS_VERSION_,'1.5','>=') )
        {
            $this->multishop_context = Shop::CONTEXT_ALL ;
            $this->id_lang = (int)Context::getContext()->language->id ;
        }
        else
        {
            $this->id_lang = (int)$cookie->id_lang ;
        }
        
        $langFile = _PS_MODULE_DIR_ . $this->module . '/' . Language::getIsoById($this->id_lang) . '.php';
      
        if (file_exists($langFile))
        {
            include($langFile);

            foreach ($_MODULE as $key => $value)
                if (preg_match('#^..' . $this->module . '#', $key))
                    $_LANGADM[str_replace('_', '', strip_tags($key))] = $value;
        }
        $this->uri = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/';

        $this->images = $this->uri . 'images/';

        // Amazon Europe
        //
        $this->amazonEurope = unserialize(base64_decode(Amazon::configurationGet('AMAZON_EUROPE')));
        $this->actives = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ACTIVE')));

        if ($this->amazonEurope && $this->marketPlaceMaster = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MASTER'))))
        {
            $marketPlaceRegion = unserialize(base64_decode(Amazon::configurationGet('AMAZON_REGION')));
            $masterPlatform = 0;


            if ($marketPlaceRegion)
                foreach ($marketPlaceRegion as $language_id => $region)
                    if ($this->marketPlaceMaster == $region)
                        break;

            $this->marketPlaceMasterId = $language_id;
        }

        else
            $this->marketPlaceMasterId = false;

        // Custom style depending PS release
        //
        if (preg_match('/^1\.[1-3]/', _PS_VERSION_))
        {
            $this->radiostyle = '';
            $this->flagstyle = 'position:relative;top:-3px;';
        }
        else
        {
            $this->radiostyle = 'position:relative;top:-2px;';
            $this->flagstyle = 'position:relative;top:-1px;';
        }

        $this->_amazon = new Amazon();

        parent::__construct();
    }

    public function l($string, $class = false, $addslashes = FALSE, $htmlentities = TRUE)
    {
        return( $this->_amazon->l($string, __CLASS__, $addslashes = FALSE, $htmlentities = TRUE) );
    }

    public function DisplayCountries()
    {
        $currencies = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CURRENCY')));
        $count = 0;
        $html = '';
        $html = '<fieldset>
        <legend><img src="../img/admin/world.gif" alt="" class="middle" />' . $this->l('Choose Amazon Platform') . '</legend>';

        if ($this->amazonEurope)
        {
            // Display Master MarketPlace
            //
            $html .='<span><b>' . $this->l('Europe') . '</b>:&nbsp;&nbsp;</span>';
            $html .= '<img src="' . $this->images . 'eu.gif" style="' . $this->flagstyle . '" alt="Europe" />&nbsp;&nbsp;';


            // Display Amazon Europe
            //
            foreach (Amazon_Tools::languages() as $language)
            {
                $id_lang = $language['id_lang'];

                // Active / Inactive
                //
                if (!isset($this->actives[$id_lang]))
                    continue;

                // Is not inside Europe Area
                //
                if (!Amazon_Tools::isEurope($language['iso_code']))
                    continue;

                $count++;
                
                $html .= '<input type="radio" name="amazon_lang" style="' . $this->radiostyle . '" value="' . $id_lang . '" rel="1" />&nbsp;&nbsp;' . "\n";
                $html .= '<img src="' . $this->images . 'geo_flags/' . $language['iso_code'] . '.gif" style="' . $this->flagstyle . '" alt="' . $language['name'] . '" />&nbsp;&nbsp;' . $language['name'] . '&nbsp;&nbsp;';
            }
        }


        // Display Other Platforms
        //
        $other = '';

        foreach (Amazon_Tools::languages() as $language)
        {
            $id_lang = $language['id_lang'];

            if (!$id_lang)
                continue;
            
            // Active / Inactive
            //
            if (!isset($this->actives[$id_lang]))
                continue;

            // Is inside Europe Area
            //
            if (Amazon_Tools::isEurope($language['iso_code']) && $this->amazonEurope)
                continue;
            
            
            // Is Master Market Place outside Euro Area
            //
            if ($this->marketPlaceMasterId == $id_lang && $currencies[$id_lang] == 'EUR')
                continue;
            
            if ( empty($other) )
                $other = '<br /><br /><span style="font-weight:bold">' . $this->l('Other Platforms') . ':</span>&nbsp;&nbsp;';
            
            $count++;
            $other .= '<input type="radio" name="amazon_lang" style="' . $this->radiostyle . '" value="' . $id_lang . '" rel="0" />&nbsp;&nbsp;';
            $other .= '<img src="' . $this->images . 'geo_flags/' . $language['iso_code'] . '.gif" style="' . $this->flagstyle . '" alt="' . $language['name'] . '" />&nbsp;&nbsp;' . $language['name'] . '&nbsp;&nbsp;';
        }
        if ($other)
            $html .= $other;

        if ( ! $count )
        {
            $html .= '<span style="color:brown;font-weight:bold;">' . $this->l('You must configure at least one platform to use the module') . '</span>' ;
        }
        
        $html .= '</fieldset><br />';

        return($html);
    }

    private function _tabs()
    {
        $selected_tab = ($tab = Tools::getValue('selected_tab')) ? $tab : 'synchronize';

       
        $this->_html .= '
              <ul id="menuTab">';

        $this->_html .= '<li id="menu-synchronize" class="menuTabButton ' . ($selected_tab == 'synchronize' ? 'selected' : '') . '"><span>&nbsp;<img src="' . $this->images . 'update.jpg" alt="' . $this->l('Synchronize Mode') . '" />&nbsp;&nbsp;' . $this->l('Synchronize') . '</span>  </li>';
        //$this->_html .= '<li id="menu-scanner" class="menuTabButton ' . ($selected_tab == 'scanner' ? 'selected' : '') . '"><span>&nbsp;<img src="' . $this->images . 'eye.png" alt="' . $this->l('ASIN Scanner') . '" />&nbsp;&nbsp;' . $this->l('ASIN Scanner') . '</span>  </li>';

        $this->_html .= '
              </ul>
        ';
    }

    public function display()
    {
        $path = pathinfo(__FILE__);

        // CSS to include
        //
        $cssFile = $this->uri . 'css/' . str_replace($path['extension'], 'css', $path['basename']);
        
        // URL to file
        // 
        $report = $this->uri . 'functions/products_report.php';
        $create_lookup = $this->uri . 'functions/product_create_lookup.php';
        
        $this->_html = '
            <div style="margin-top:50px;">
            <img style="float:right;" src="' . $this->images . 'amazon.png" alt="' . $this->l('Amazon Market Place') . '" />
            <div class="clean" style="padding-bottom:50px;"></div>
            <form action="' . $_SERVER['REQUEST_URI'] . '" method="post" id="updateAmazon">
            <input type="hidden" id="id_lang" value="' . $this->id_lang . '" />    
            <input type="hidden" id="report_url" value="' . $report . '" />                
            <input type="hidden" id="product_create_lookup_url" value="' . $create_lookup . '" />                
            <input type="hidden" id="css_file" value="' . $cssFile . '" />            
        ' ;

        $this->_html .= $this->DisplayCountries() ;
        
        $this->_html .= $this->_tabs();

        
        $this->_html .= '
        <div id="tabList">
        ';

        $this->_html .= $this->synchronize();
        
        //$this->_html .= $this->scanner();

        $this->_html .= '
         </form>   
        </div>
        </div>';

        // jQuery Functions
        //
        $this->_html .= '<script type="text/javascript" src="' . $this->uri . 'js/products.js?path=' . $this->uri . '"></script>';        
        
        echo $this->_html;
    }

    private function scanner()
    {
        global $cookie;

        $selected_tab = ($tab = Tools::getValue('selected_tab')) ? $tab : 'synchronize';
        
        $html = '' ;
        $html .= '
                <div id="menudiv-scanner"  class="tabItem ' . ($selected_tab == 'create' ? 'selected' : '') . '" rel="scanner">' ;

        $html .= '<br /><br />
                <fieldset>
                <input type="hidden" id="scanner_url" value="' . $this->uri . 'functions/scanner.php" />
                <input type="hidden" id="scanner_products_ok" value="' . $this->l('The products have been loaded successfuly') . '" />
                <legend><img src="' . $this->images . 'eye.png" alt="" class="middle" />' . $this->l('ASIN Scanner') . '</legend>
                <p><b>' . $this->l('Scan Amazon to get the ASINs corresponding to your catalog') . ' </b></p>
                <p>' . $this->l('The inexisting product will be created on the next synchronisation') . ' </p>
                <p>' . $this->l('You must start this scan for each country') . ' </p>        
                <p>' . $this->l('If you have many products in your catalog may be you will have to start this scan more than one time') . ' </p><br />' ; 
        $html .= '<input type="button" class="button" style="float:right" id="asin-scanner" value="' . $this->l('Scan for ASIN') . '" />' ;
        $html .= '<br /><br />' ;
        
        $html .= '<div class="conf" id="scanner-report" style="display:none;width:800px"></div>' ;
        
        $html .= '</fieldset><br /></div>' ;
        return($html) ;
    }
    private function synchronize()
    {
        global $cookie;

        $selected_tab = ($tab = Tools::getValue('selected_tab')) ? $tab : 'synchronize';
        
        $loader = $this->images . 'loading.gif';

        $export = $this->uri . 'functions/products.php';

        $oldest = Amazon_Product::oldest();

        $initialDate = date('Y-m-d H:i:s', strtotime($oldest));
        $currentDate = date('Y-m-d H:i:s');
        $today = date('Y-m-d H:i:s', strtotime('today midnight'));

        $dateWS = Amazon::configurationGet('AMAZON_LAST_UPDATED_' . $this->id_lang);
        $dateCron = Amazon::configurationGet('AMAZON_LAST_CRON_UPDATED_' . $this->id_lang);

        $html = '
        <div id="menudiv-synchronize"  class="tabItem ' . ($selected_tab == 'synchronize' ? 'selected' : '') . '" rel="synchronize">
        <div style="clear:both;">
                <input type="hidden" id="update_url" value="' . $export . '" />
                <input type="hidden" id="msg_lang" value="' . $this->l('Please choose an Amazon platform') . '" />
                <input type="hidden" id="msg_date" value="' . $this->l('You have to choose a date range') . '" />
                <input type="hidden" id="img_loader" value="' . $loader . '" />
                <input type="hidden" name="currentDate" value="' . $currentDate . '" /></div><br />
                <input type="hidden" name="id_lang" value="' . $this->id_lang . '" />
            ';

        $amazonTokens = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CRON_TOKEN')));

        if ( $amazonTokens )
        foreach ($amazonTokens as $id_lang => $token)
        {
            $html .= '
                <input type="hidden" name="amazon_token[' . $id_lang . ']" value="' . $token . '" />';
        }

        //
        // Options
        //       
        $html .= 
        '<br />

            <fieldset>
            <legend><img src="' . $this->images . 'cog.gif" alt="" class="middle" />' . $this->l('Options') . '</legend>
                <table width="100%">
                <tr>
                    <td>
                        <input type="checkbox" value="1" name="active_only" checked="true" /><span class="send-actives">' . $this->l('Active Only') . '</span>
                    </td>
                    <td>
                        <input type="checkbox" value="1" name="synch-descriptions" /><span class="text-synch-desc">' . $this->l('Send Titles/Descriptions') . '</span>&nbsp;&nbsp;&nbsp;
                    </td>
                    <td>
                        <span class="option-ext-item">
                            <input type="checkbox" value="1" name="delete" /><span class="delete">' . $this->l('Delete') . '</span>&nbsp;   
                        </span>                        
                    </td>
                </tr>
                <tr>
                    <td>
                        <input type="checkbox" value="1" name="create" /><span class="create">' . $this->l('Create Products') . '</span>
                    </td>
                    <td>
                        <input type="checkbox" value="1" name="images" /><span class="send-images">' . $this->l('Send Images') . '</span>
                    </td>
                    <td> 
                        <span class="option-ext-item">
                            <input type="checkbox" value="1" name="relations-only" /><span class="relations-only">' . $this->l('Send Relations Only') . '</span>&nbsp;   
                        </span>
                        
                    </td>
                </tr>
                <tr>
                    <td>
                        <span class="option-ext-item">
                            <input type="checkbox" value="1" name="xml-only" /><span class="xml-only">' . $this->l('Display XML then Exit') . '</span>&nbsp;   
                        </span>
                    </td>
                    <td>
                    </td>
                    <td> 
                    </td>
                </tr>
                
                </table>
                <span class="options-extended">...</span>
                <input type="hidden" value="' . $this->l('A server-side error has occurred. Please contact you server administrator') . '" id="serror" />
           </fieldset> ';

        $stats = Amazon_Product::marketplaceCountProducts() ;

        $actives = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ACTIVE')));
        
        $html .= '<br />
            <fieldset>
            <p>
                <b> ' . $this->l('Statistics') . '</b>: ' . sprintf($this->l('There is %d synchronizable products and %d combinations in %d languages'), $stats['products'], $stats['attributes'], count($actives)) . '
                <br /><br />
                <b>' . $this->l('Latest Updates') . '</b>:
                    
                <table style="margin-left:40px;width:600px;">
                <tr style="font-style:italic"><td width="40%">' . $this->l('Platform') . '</td><td>' . $this->l('Cron Update') . '</td><td></td><td>' . $this->l('Normal Update') . '</td></tr>'; 

        $count = 0 ;
        if ( $actives )
        foreach( array_keys($actives) as $selected_lang )
        {
            $region = Amazon_Tools::idToDomain($selected_lang) ;

            $dateWS = Amazon::configurationGet('AMAZON_LAST_UPDATED_' . $selected_lang);
            $dateCron = Amazon::configurationGet('AMAZON_LAST_CRON_UPDATED_' . $selected_lang);

            if ( $dateWS )
                $strWS = Tools::displayDate($dateWS, $this->id_lang, true) ;
            else 
                $strWS = $this->l('Never') ;
            
            if ( $dateCron )
                $strCron = Tools::displayDate($dateCron, $this->id_lang, true) ;
            else 
                $strCron = $this->l('Never') ;

            $count++ ;
            
            $html .= '<tr><td>Amazon.' . $region . '</td><td>' . $strCron . '</td><td style="width:40px"></td><td>' . $strWS . '</td></tr>' ;
        }  
        $html .= '
                </table>' ;
        
        if ( ! $count )
        {
            $html .= '<span style="color:brown;font-weight:bold;">' . $this->l('No recorded updates') . '</span>' ;
        }
        
        $default_lang = strtolower(Language::getIsoById(Amazon::configurationGet('PS_LANG_DEFAULT'))) ;
        
        if ( $default_lang == 'fr' )
        {
            $documentation = $this->uri . 'documentation/documentation.pdf' ;
            $faq = $this->uri . 'documentation/faq_fr.pdf' ;
        }
        else
        {
            $documentation = $this->uri . 'documentation/documentation-en.pdf' ;
            $faq = $this->uri . 'documentation/faq_en.pdf' ;
        }
        $documentation_link = '[ <a href="' . $documentation . '" class="doclink" target="_blank">' . $this->l('Documentation') . '</a> ]' ;
        $faq_link = '[ <a href="' . $faq . '" class="doclink" target="_blank">' . $this->l('FAQ') . '</a> ]' ;

        $html .= '
                    <hr style="width:50%" />' . 
                $this->l('Here are the main informations to help you to synchronize your catalog') . ' :<br /><br />' .
                
                '<u>' . $this->l('On the left :') . '</u><br /><br />' .
                
                $this->l('Send: will send all the products within the selected categories') . '<br /><br />' . 

                $this->l('Verify: Execute the same action, but for checking. This option will send nothing') . '<br /><br />' .
                
                $this->l('This is the method to send your entire catalog the first time') . '<br /><br />' . 
                
                $this->l('This is also the method the usual method to create products sheets (which require to have setted up a profile for the category)') . '<br /><br />' . 
                
                '<u>' . $this->l('On the right :') . '</u><br /><br />' .
                
                $this->l('Update stocks: will synchronize Amazon depending your stocks moves: Products Added, Updated, Deleted') . '<br /><br />' . 
                
                $this->l('Verify: Execute the same action than update stocks, but for checking. This option will send nothing') . '<br /><br />' . 

                '<u>' . $this->l('Please Note :') . '</u><br /><br /><span style="color:brown">' .
                
                $this->l('Do not forget that the informations sent to Amazon will be computed with a delay') . '<br /><br />' . 
                
                $this->l('Please wait a moment and ask for a report with the appropriate button') . '</span><br /><br />' . 
                
                $this->l('In case of doubt about the result or non-result please read the documentation, FAQ or ask the support') .
                
                '.&nbsp;&nbsp;' . $documentation_link . '&nbsp;-&nbsp;' . $faq_link . '<br /><br />' . 
                
                '<br /><hr style="width:50%" />' ;

        @includeDatepicker(array('send_since')) ;
        //
        // Submit
        //
        $dateSince = preg_replace('/\s.*/', '', str_replace('/', '-', $oldest)) ;
        $html .= '
            <span style="float:left;color:grey;">
                <input type="submit" name="verify" value="' . $this->l('Verify') . '" class="button" />&nbsp;
                 <input type="submit" name="export" value="' . $this->l('Send') . '" class="button" />&nbsp;'
                . $this->l('The Entire Catalog Since :') . '
                <input type="text" style="width:80px;" id="send_since" name="since" value="' . $dateSince . '" />
            </span>
            
            <input type="submit" name="update" value="' . $this->l('Update Stocks') . '" class="button" style="float:right" />
            <input type="submit" name="lookup" value="' . $this->l('Verify') . '" class="button" style="float:right;margin-right:10px" />    

            </p>    
            </fieldset>
            <br />
        ';

        $html .= '<div class="conf" id="result" style="display:none;width:90%;"></div>' ;
        $html .= '<div class="error" id="result" style="display:none;width:90%;"></div>' ;
        
        //
        // Reports
        //
        if (Amazon::configurationGet('AMAZON_LAST_SUBMISSIONS'))
            $display = '';
        else
            $display = 'display:none; ';


        $html .= '<br />

            <fieldset id="report-set" style="' . $display . '">
            <legend><img src="../img/admin/printer.gif" alt="" class="middle" />' . $this->l('Reports') . '</legend>
            <div id="wait-report" style="display:none;clear:both;width:250px;line-height:200%;background-color:#fefefe;border:1px solid silver;">&nbsp;&nbsp;<img src="' . $this->images . 'small-loader.gif" style="position:relative;top:-7px;" alt="" />&nbsp;&nbsp;<span>' . $this->l('Please wait for the report') . '</span></div><br />
            <input type="submit" id="request-submission-result" value="' . $this->l('Display Export Result') . '" class="button" style="float:right;" />
            <div id="submission-results">

            </fieldset>
                    </form><br />
            <div class="conf" id="amazon-report" style="display:none;width:90%"></div>
         </div>';

        return($html);
    }

}

?>