<?php



require_once(dirname(__FILE__) . '/../../../config/config.inc.php');
require_once(dirname(__FILE__) . '/../../../init.php');

require_once(dirname(__FILE__) . '/../amazon.php');

require_once(dirname(__FILE__) . '/../classes/amazon.product.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.address.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.order.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.orderhistory.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.cart.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.payment.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.webservice.class.php');

global $cookie;

class AmazonImportOrder extends Module
{
    private $_amazonApi = array();
    private $_debug = false;
    private $module = 'amazon';
    public $name = 'amazon';
    private $_amazon;
    public $id_lang;

    function __construct()
    {
        global $cookie, $_MODULE, $_LANGADM;

        $this->id_lang = (int)Tools::getValue('id_lang');
        
        if ( ! $this->id_lang ) 
            $this->id_lang = Configuration::get('PS_LANG_DEFAULT') ;
        
        $langFile = _PS_MODULE_DIR_ . $this->module . '/' . Language::getIsoById($this->id_lang) . '.php';
        $mySelf = basename(__FILE__, '.php');

        if (file_exists($langFile))
        {
            require_once $langFile;
            foreach ($_MODULE as $key => $value)
                $_MODULE[str_replace($mySelf . '_', $this->name . '_', $key)] = $value;
        }
        $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/';
        $this->images = $module . 'images/';


        $this->_amazon = new Amazon();

        parent::__construct();
    }

    public function l($string, $specific = false, $id_lang = null)
    {
        return( $this->_amazon->l($string, $specific = false, $id_lang = null) );
    }

    public function Import()
    {
        global $cookie, $cronMode, $europe, $id_lang, $status, $order_id, $currentDate, $cr, $date1, $date2;

        $i = 0;

        $tokenOrders = Tools::getValue('token_order');

        $debugMode = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG')));
        $this->_debug = $debugMode ? true : false;

        $id_warehouse = null;
        $id_shop = 1;

        // Shop Configuration (PS 1.5)
        // 
        if (version_compare(_PS_VERSION_, '1.5', '>='))
        {
            $id_shop = (int) Amazon::configurationGet('AMAZON_SHOP');
            $id_warehouse = (int) Amazon::configurationGet('AMAZON_WAREHOUSE');

            if (!$id_shop)
                $id_shop = 1;
        }

        // Carriers
        //
        $carriers = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CARRIER')));
        $amazon_carriers = unserialize(base64_decode(Amazon::configurationGet('AMAZON_AMAZON_CARRIER')));

        // Currency
        //
        $currencies = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CURRENCY')));

        // To access to the module properties
        //
        $amazon = new Amazon();

        // Fulfillment by Amazon (FBA)
        //
        $FBA = Amazon::configurationGet('AMAZON_FBA');

        // Order State
        //
        $idOrderState = array();
        $idOrderState[Amazon::MFN] = intval(unserialize(base64_decode(Amazon::configurationGet('AMAZON_ORDER_STATE'))));
        $idOrderState[Amazon::AFN] = intval(Amazon::configurationGet('AMAZON_FBA_ORDER_STATE'));

        // Use Taxes - Amazon Config overrides PS Config
        //
        $useTaxes = false;

        if (Tax::excludeTaxeOption())
            $useTaxes = false;

        if (intval(unserialize(base64_decode(Amazon::configurationGet('AMAZON_TAXES')))))
            $useTaxes = true;

        // Force import on out of stock
        //
      $force_import = unserialize(base64_decode(Amazon::configurationGet('AMAZON_FORCE_IMPORT')));
        $force_import = intval($force_import) ? true : false;

        // Import unknown products as a dummy product
        //
        $dummy = (int) Amazon::configurationGet('AMAZON_DUMMY');
        $dummy_product = Amazon::configurationGet('AMAZON_DUMMY_PRODUCT');

        // Add Region to the payment title (UK, FR, US etc..)
        //
        $paymentRegion = Amazon::configurationGet('AMAZON_PAYMENT_REGION') ? true : false;

        // Amazon Europe
        $this->europe = $europe;

        // Init
        //
        $amazon = Amazon_Tools::selectPlatforms($id_lang, $this->_debug);


        if ($this->_debug)
            echo nl2br(print_r($amazon['auth'], true) . print_r($amazon['params'], true) . print_r($amazon['platforms'], true));

        if (!$this->_amazonApi = new Service($amazon['auth'], $amazon['params'], $amazon['platforms'], $this->_debug, "<br />\n"))
        {
            echo $this->l('Unable to login') . $cr;
            die;
        }

        // Amazon Throtlling Management
        // 
        $this->_amazonApi->setTimerReadHandler('Configuration', 'get', 'AMAZON_API_TIMER');
        $this->_amazonApi->setTimerWriteHandler('Configuration', 'updateValue', 'AMAZON_API_TIMER');
        // Do not drop in this context
        // $this->_amazonApi->setTimerAction( Service::DROP_THROTLLED_QUERIES ) ;

        if ($cronMode)
        {
            $amazonOrders = null;
            echo $date1 . $cr;
            echo $date2 . $cr;
            if (!$amazonOrders = $this->_amazonApi->GetUnshippedOrdersList($date1, $date2, $status, $FBA))
            {
                printf($this->l('No orders fetched from Amazon') . $cr);
                die;
            }
        }
        else
        {
            $orders_ids = Tools::getValue('order_id');

            if (!count($orders_ids))
            {
                printf($this->l('Nothing to import...') . $cr);
                die;
            }

            if (!$amazonOrders = $this->_amazonApi->GetOrders($orders_ids))
            {
                printf($this->l('No orders fetched from Amazon') . $cr);
                die;
            }
        }
        // Regions
        //
        $marketPlaceRegion = unserialize(base64_decode(Amazon::configurationGet('AMAZON_REGION')));

        // Identify the order's country
        //
        $marketPlaceIds = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MARKETPLACE_ID')));
        $marketPlace2IsoId = array_flip($marketPlaceIds);

        foreach ($amazonOrders as $key => $order)
        {
            $order_id = (string) $order->AmazonOrderId;

            // Charset conversion 
            //
            foreach ($order as $order_key => $order_item)
            {
                if (!is_object($order->$order_key) && !is_array($order->$order_key) && strval($order->$order_key))
                    $order->$order_key = iconv("ISO-8859-15", "UTF-8//TRANSLIT", $order_item);
            }

            // Skipping Cancelled Orders
            //
            if ((string) $order->OrderStatus == 'Canceled')
            {
                echo basename(__FILE__) . ': ' . __LINE__ . ' - ' . $this->l('Skipping Cancelled Order') . ' #' . $order_id;
                continue;
            }
            if ($this->_debug)
                echo nl2br(print_r($order, true));

            if (Amazon_Order::checkByMpId($order_id))
            {
                printf($this->l('Order ID (%s) Was already imported...') . $cr, $order_id);
                continue;
            }

            if (!isset($order->Address) || !is_object($order->Address) ||
                    !isset($order->Address->Name))
            {
                echo basename(__FILE__) . ': ' . __LINE__ . ' - ' . $this->l('Couldn\'t import incomplete order') . '[' . print_r($order, true) . ']' . $cr;
                continue;
            }

            if (empty($order->Address->Name) || empty($order->Address))
            {
                echo basename(__FILE__) . ': ' . __LINE__ . ' - ' . $this->l('Couldn\'t import incomplete order') . '[' . print_r($order->Address, true) . ']' . $cr;
                continue;
            }

            if (!isset($order->BuyerEmail) || empty($order->BuyerEmail))
            {
                echo basename(__FILE__) . ': ' . __LINE__ . ' - ' . $this->l('Missing Buyer Email for this order') . $cr;
                continue;
            }
            $idCurrency = (int) Currency::getIdByIsoCode($order->OrderTotalCurrency);

            // Langue de la Commande
            //
            if (isset($marketPlace2IsoId[$order->MarketPlaceId]))
                $id_lang = $marketPlace2IsoId[$order->MarketPlaceId];

            // Amazon Region (fr, de, ...)
            // 
            if (isset($marketPlaceRegion[$id_lang]))
                $region = strtoupper($marketPlaceRegion[$id_lang]);
            else
                $region = null;

            // Amazon Global Customer Account ID
            //
            $id_customer = Amazon::configurationGet('AMAZON_CUSTOMER_ID');

            $account_type = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ACCOUNT_TYPE')));

            // Customer individual account
            //
            if ($account_type == Amazon::AccounTypeIndividual && isset($order->BuyerEmail) && !empty($order->BuyerEmail))
            {
                $email_address = (string) $order->BuyerEmail;

                $customer = new Customer();

                if (empty($email_address))
                {
                    echo basename(__FILE__) . ': ' . __LINE__ . ' - ' . $this->l('Couldn\'t add this customer') . ' : ' . $name . '(' . $email_address . ')';
                    continue;
                }
                $customer->getByEmail($email_address);

                if ($customer->id)
                {
                    $id_customer = $customer->id;
                }
                else
                {
                    $email_address = (string) $order->BuyerEmail;

                    $name = Amazon_Address::getAmazonName((string) $order->Address->Name);

                    $customer->firstname = $name['firstname'];
                    $customer->lastname = $name['lastname'];
                    $customer->email = $email_address;
                    $customer->passwd = md5(rand());

                    if (!$customer->add())
                    {
                        echo $this->l('Couldn\'t add this customer') . ' : ' . $name . '(' . $email_address . ')';
                    }
                    else
                    {
                        $id_customer = $customer->id;
                    }
                }
            }

            // FROM AMZON to PRESTASHOP !
            //
            $shipping_address = new Amazon_Address();
            $shipping_address->id_customer = $id_customer;
            $shipping_address_id = $shipping_address->lookupOrCreateAmazonAddress($order->Address, $order->BuyerEmail);

            $created_at_time = strtotime($order->PurchaseDate);
            $created_at = Tools::displayDate(date('Y-m-d H:i:s', strtotime($order->PurchaseDate)), $id_lang, true);
            $date_add = date('Y-m-d H:i:s', strtotime($order->PurchaseDate));
            $fees = 0;
            $irow = 0;
            $shipping_price = 0;
            $channel = (string) $order->FulfillmentChannel;

            // Associate Shipping Method
            //
            $shipping_method = trim((string) $order->ShipServiceLevel);

            for ($i = 0, $id_carrier = 0; $i < count($amazon_carriers[$id_lang]); $i++)
            {
                if (md5($shipping_method) == $amazon_carriers[$id_lang][$i])
                    $id_carrier = $carriers[$id_lang][$i];
            }

            if (!$id_carrier)
            {
                echo '<br /><span style="color:brown;">';
                printf($this->l('Unable to associate the carrier (%s) for order #%s') . $cr, $shipping_method, $order->AmazonOrderId);
                echo '<br />' . "\n";
                printf('%s', $this->l('This is not an error ! You must configure the carrier mapping in the module configuration...'));
                echo '</span><hr />';
                continue;
            }

            if (!$Items = $this->_amazonApi->getOrderItems($order_id))
            {
                echo $this->l('Unable to retrieve items for Amazon Order') . ': ' . $order_id . $cr;
                echo '<hr />';
                continue;
            }

            if (!$cronMode)
            {
                echo '<h3>' . $this->l('Order ID') . ': ' . $order->AmazonOrderId . '</h3>';

                echo '<table width="100%" class="table">';

                echo '<thead>' .
                '<th>' . $this->l('Reference') . '</th>' .
                '<th>' . $this->l('ASIN') . '</th>' .
                '<th>' . $this->l('Product Name') . '</th>' .
                '<th>' . $this->l('Qty') . '</th>' .
                '<th>' . $this->l('Currency') . '</th>' .
                '<th>' . $this->l('Total') . '</th>' .
                '</thead>';
            }
            else
            {
                echo $this->l('Order ID') . ': ' . $order->AmazonOrderId . $cr;
                echo $this->l('Reference') . "\t";
                echo $this->l('ASIN') . "\t";
                echo $this->l('SKU') . "\t";
                echo $this->l('Product Name') . "\t";
                echo $this->l('Qty') . "\t";
                echo $this->l('Currency') . "\t";
                echo $this->l('Total') . "\t";
            }
            $itemDetails = array();


            // Building Cart
            //
            $cart = new Amazon_Cart();
            $cart->id_address_delivery = $shipping_address_id;
            $cart->id_address_invoice = $shipping_address_id;
            $cart->id_carrier = $id_carrier;
            $cart->id_currency = $idCurrency;
            $cart->id_customer = $id_customer;
            $cart->id_lang = $id_lang;
            $cart->add();

            $totalQuantity = $totalSaleableQuantity = 0;

            $shipping_method = 0;

            $status = strval($order->OrderStatus);
            $mpStatusId = constant("Amazon_Order::$status");

            $i = 0;
            foreach ($Items as $key => $item)
            {
                $quantity = intval($item->QuantityOrdered);
                $price = floatval($item->ItemPriceAmount) / $quantity;
                $totalQuantity += intval($item->QuantityOrdered);
                $product_name = strval($item->Title);
                $SKU = trim(strval($item->SKU));

                $shipping_price += floatval($item->ShippingPriceAmount);

                if (!$cronMode)
                {
                    printf('
                        <tr class="' . ($irow++ % 2 ? 'alt_row' : '') . '">
                        <td>%s</td>
                        <td>%s</td>
                        <td style="width:300px">%s</td>
                        <td align="right">%s</td>
                        <td align="right">%s</td>
                        <td align="right">%s</td>
                        </tr>
                  ', $SKU, $item->ASIN, $product_name, $quantity, $item->ItemPriceCurrency, Tools::displayPrice($price * $quantity, $idCurrency)
                    );
                }
                else
                {
                    printf("%s\t%s\t%s\t%s\t%s\t%s\t%s\n", $SKU, $item->ASIN, $item->SKU, $product_name, $quantity, $item->ItemPriceCurrency, Tools::displayPrice($price * $quantity, $idCurrency)
                    );
                }

                $product = new Amazon_Product($SKU, true, $id_lang);

                $id_product = isset($product->id) ? $product->id : null;

                $dummy_import = false;
                $realSKU = $SKU;

                if (!$id_product && $dummy)
                {
                    $product = new Amazon_Product($dummy_product, true, $id_lang);

                    $id_product = isset($product->id) ? $product->id : null;

                    $SKU = $dummy_product;

                    $dummy_import = true;
                }

                if (!$id_product)
                {
                    echo '<br /><span style="color:red;">';
                    printf($this->l('Unable to find the expected product for order #%s product ASIN: %s SKU: %s') . $cr, $order->AmazonOrderId, $item->ASIN, $SKU);
                    unset($itemDetails[$SKU]);
                    echo '</span>';
                    continue;
                }

                // Two products but with the same reference !
                //
              if (isset($itemDetails[$SKU]))
                {
                    $itemDetails[$SKU]['qty'] += $quantity;
                }
                else
                {
                    $itemDetails[$SKU] =
                            array(
                                'id'              => $SKU,
                                'qty'             => $quantity,
                                'price'           => $price,
                                'name'            => $product_name,
                                'dummy'           => $dummy_import,
                                'real_sku'        => $realSKU,
                                'shipping'        => 0,
                                'fees'            => 0
                            );
                }
                if ($useTaxes)
                // PS 1.4 sinon 1.3
                    //
                if (method_exists('Tax', 'getProductTaxRate'))
                        $product_tax_rate = (float) (Tax::getProductTaxRate($product->id, $shipping_address_id));
                    else
                        $product_tax_rate = (float) (Tax::getApplicableTax($product->id_tax, $product->tax_rate, $shipping_address_id));
                else
                    $product_tax_rate = 0;

                // Modif YB
                $itemDetails[$SKU]["tax_rate"] = $product_tax_rate;
                $itemDetails[$SKU]["id_tax"] = isset($product->id_tax) ? $product->id_tax : false;
                $itemDetails[$SKU]["id_product"] = $product->id;
                $itemDetails[$SKU]["id_address_delivery"] = $shipping_address_id;

                // Product Combinations
                //
                if (version_compare(_PS_VERSION_, '1.5', '<'))
                    $combinations = $product->getAttributeCombinaisons($id_lang);
                else
                    $combinations = $product->getAttributeCombinations($id_lang);

                $id_product_attribute = 0;

                if ($combinations)
                    foreach ($combinations as $key => $combination)
                        if (trim($combination['reference']) == $SKU)
                        {
                            $id_product_attribute = intval($combination['id_product_attribute']);
                            $itemDetails[$SKU]['id_product_attribute'] = $id_product_attribute;
                        }

                if (version_compare(_PS_VERSION_, '1.5', '<'))
                {
                    $productQuantity = Product::getQuantity(intval($id_product), $id_product_attribute ? $id_product_attribute : null);
                }
                else
                {
                    $productQuantity = Product::getRealQuantity($id_product, $id_product_attribute ? $id_product_attribute : null, $id_warehouse, $id_shop);
                }

                $options = Amazon_Product::getProductOptions($id_product, $id_lang);

                $updateStocks = true;

                // Handling the order as Fulfilled by Amazon - do not update stocks levels
                //
                if ($channel == Amazon::AFN)
                    $updateStocks = false;

                // Dummy Product - do not update stocks levels
                //
              if ($dummy_import)
                    $updateStocks = false;

                if ($updateStocks)
                {
                    $force = $options['force'] ? true : false;

                    if (!($force || $force_import) && $productQuantity - $quantity < 0)
                    {
                        printf('%s ASIN: %s SKU: %s' . $cr, $this->l('Not enough stock to import this product'), $item->ASIN, $SKU);
                        unset($itemDetails[$SKU]);
                        continue;
                    }
                    elseif (($force || $force_import) && $productQuantity - $quantity < 0)
                    {
                        if (version_compare(_PS_VERSION_, '1.5', '>='))
                        {
                            StockAvailable::updateQuantity($id_product, $id_product_attribute ? $id_product_attribute : null, $quantity);
                        }
                        else
                        {
                            Amazon_Product::updateProductQuantity($id_product, $id_product_attribute ? $id_product_attribute : null, $quantity);
                        }
                    }

                    if ($cart->updateQty($quantity, $id_product, $id_product_attribute) < 0)
                    {
                        echo $this->l('Couldn\'t update cart quantity: not enough stock ?') . ' (ASIN:' . $item->ASIN . ' SKU:' . $SKU . ')' . $cr;
                        unset($itemDetails[$SKU]);
                        continue;
                    }
                }
                else // Handling the order as Fulfilled By Amazon
                {
                    $cart->updateQty($quantity, $id_product, $id_product_attribute);
                }
                $i++;

                $totalSaleableQuantity += intval($item->QuantityOrdered);
            }

            if ($totalQuantity != $totalSaleableQuantity)
            {
                echo $this->l('Skipping Order: Product count mismatch') . ' (' . $order->AmazonOrderId . ')' . $cr;

                if (Validate::isLoadedObject($cart))
                    $cart->delete();

                if (!$cronMode)
                    echo '<hr />';

                continue;
            }

            if (!count($itemDetails))
            {
                echo $this->l('Skipping Order: No products for this order') . ' (' . $order->AmazonOrderId . ')' . $cr;

                if (Validate::isLoadedObject($cart))
                    $cart->delete();

                if (!$cronMode)
                    echo '<hr />';

                continue;
            }

            if (!$cronMode)
                echo '</table>';

            // Using price, shipping details etc... from the Market Place
            //
            $cart->amazonProducts = $itemDetails;
            $cart->amazonShipping = $shipping_price;
            $cart->amazonChannel = $channel;

            if ($this->_debug)
                printf('Amazon Cart: %s', nl2br(print_r($cart->amazonProducts, true)));

            // Payment Title
            //
            if ($paymentRegion)
                $paymentTitle = trim(sprintf('%s %s', Amazon::PaymentMethodTitle, $region));
            else
                $paymentTitle = Amazon::PaymentMethodTitle;

            // duplication du panier, important !!!
            //
            $acart = $cart;

            $payment = new Amazon_PaymentModule();

            if (!$i)
            {
                echo $this->l('Skipping Order: No products for this order') . ' (' . $order->AmazonOrderId . ')' . $cr;

                if (Validate::isLoadedObject($cart))
                    $cart->delete();

                continue;
            }
            elseif ( ($newOrderId = $payment->validateMarketplaceOrder($cart->id, $idOrderState[$channel], $paymentTitle, $order_id, $mpStatusId, $acart, $useTaxes, $date_add)) )
            {
                echo '<br />';
                printf('<span style="color:green;display:inline-block;margin-bottom:5px;">' . $this->l('Order ID %s(%s) Successfully imported in the Order Tab') . '</span>' . $cr, $order_id, $newOrderId);

                printf('%s: <img src="%sdetails.gif" style="cursor:pointer;" onclick="window.open(\'?tab=AdminOrders&id_order=%s&vieworder&token=%s\')">', $this->l('Go to the order in a new window'), $this->images, $newOrderId, $tokenOrders);
            }
            else
            {
                echo $this->l('Error while importing this order ID') . ': ' . $order->AmazonOrderId . $cr;

                if (Validate::isLoadedObject($cart))
                    $cart->delete();
            }
            if (!$cronMode)
                echo '<hr />';
        }
        if ($cronMode && $i)
            Amazon::configurationUpdateValue('AMAZON_ORDERS_RETRIEVED', $currentDate);
    }

    public function Dispatch($action)
    {
        switch ($action)
        {
            default :
                $this->Import();
        }
    }

}

$cronMode = 0;
$currentDate = date('Y-m-d H:i:s');
$amazonEurope = 0;

// WGET MODE
if (Tools::getValue('cron'))
{
    $lang = Tools::getValue('lang');
    $cronToken = Tools::getValue('cron_token');

    $cr = "<br />\n"; // carriage return

    $cronMode = 1;
    echo "Starting Order Query in WS API/Cron Mode" . ' - ' . $currentDate . $cr;

    $date1 = date('c', ($storedDate = strtotime(Amazon::configurationGet('AMAZON_ORDERS_RETRIEVED'))));

    if ($storedDate < strtotime($currentDate))
        $date1 = date('c', strtotime('now - 1 day')); // first time ...

    $date2 = date('c', strtotime('now - 15 min'));

    echo "Fetching orders from $date1 to $date2 " . $cr;

    if (intval(Tools::getValue('europe')))
        $europe = 1;
    else
        $europe = 0;

    // Regions
    //
    $marketPlaceRegion = unserialize(base64_decode(Amazon::configurationGet('AMAZON_REGION')));
    $marketLang2Region = array_flip($marketPlaceRegion);

    if (!isset($marketLang2Region[$lang]) || !$marketLang2Region[$lang])
        die($this->l('No selected language, nothing to do...'));

    $id_lang = $marketLang2Region[$lang];

    if (!Amazon_Tools::checkToken($cronToken, $id_lang, $amazonEurope))
        die('Wrong Token');

    $status = array();
    $status = 'Unshipped';
}
//
// Web Mode
//
else
{
    $cr = "<br />\n"; // carriage return

    echo "Starting Order Query in WS API/Web Mode" . ' - ' . $currentDate . $cr;

    $status = strval(Tools::getValue('statuses'));
    $amazonEurope = strval(Tools::getValue('europe'));
    $id_lang = intval(Tools::getValue('amazon_lang'));

    if (intval(Tools::getValue('europe')))
        $europe = 1;
    else
        $europe = 0;

    $tokens = Tools::getValue('amazon_token');

    if (!Amazon_Tools::checkToken($tokens, $id_lang, $amazonEurope))
        die('Wrong Token');

    if (!intval($id_lang))
        die($this->l('No selected language, nothing to do...'));
}

$amazonImportOrder = new AmazonImportOrder();
$amazonImportOrder->Dispatch('');
?>