<?php

require_once(dirname(__FILE__) . '/../../../config/config.inc.php');

require_once(dirname(__FILE__) . '/../amazon.php');

require_once(dirname(__FILE__) . '/../classes/amazon.product.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.address.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.order.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.orderhistory.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.cart.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.payment.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.webservice.class.php');

class AmazonListOrder extends Module
{

    private $_amazonApi = array();
    private $_debug = false;
    private $module = 'amazon';
    public $name = 'amazon';
    private $_amazon;

    function __construct()
    {
        global $cookie, $_MODULE, $_LANGADM;

        $this->id_lang = (int)Tools::getValue('id_lang');

        if ( ! $this->id_lang ) 
            $this->id_lang = Configuration::get('PS_LANG_DEFAULT') ;
        
        $langFile = _PS_MODULE_DIR_ . $this->module . '/' . Language::getIsoById($this->id_lang) . '.php';
        $mySelf = basename(__FILE__, '.php');

        if (file_exists($langFile))
        {
            require_once $langFile;
            foreach ($_MODULE as $key => $value)
                $_MODULE[str_replace($mySelf . '_', $this->name . '_', $key)] = $value;
        }
        $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/';
        $this->images = $module . 'images/';

        $debugMode = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG')));
        $this->_debug = intval($debugMode) ? true : false;

        $this->_amazon = new Amazon();

        parent::__construct();
    }

    public function l($string, $specific = false, $id_lang = null)
    {
        return( $this->_amazon->l($string, $specific = false, $id_lang = null) );
    }

    public function Dispatch($action)
    {
        switch ($action)
        {
            case 'list' :
                $this->DisplayList();
        }
    }

    public function DisplayList()
    {
        global $cookie;

        $cronMode = 0;
        $currentDate = date('Y-m-d H:i:s');

        $cr = "<br />"; // carriage return

        echo $this->l("Starting Order Query in WS API/Web Mode") . ' - ' . $currentDate . $cr;

        $status = strval(Tools::getValue('statuses'));

        $date1 = str_replace('-', '/', Tools::getValue('datepickerFrom'));
        $date2 = str_replace('-', '/', Tools::getValue('datepickerTo'));

        // Today - 5 minutes de temps de latence afin d'�viter les erreur de synchro dus a l'heure systeme
        //
      if (date('Ymd', strtotime($date2)) >= date('Ymd') || empty($date2))
        {
            $date1 = date('c', strtotime($date1));
            $date2 = date('c', strtotime('now - 5 min'));
        }
        elseif (date('Ymd', strtotime($date1)) >= date('Ymd', strtotime($date2)))
        {
            $date1 = date('c', strtotime($date1 . ' 00:00'));
            $date2 = date('c', strtotime($date2 . ' 23:59'));
        }
        else
        {
            $date1 = date('c', strtotime($date1 . ' 00:00'));
            $date2 = date('c', strtotime($date2));
        }

        $id_lang = intval(Tools::getValue('amazon_lang'));
        $tokenOrders = Tools::getValue('token_order');

        if (!intval($id_lang))
        {
            echo $this->l('No selected language, nothing to do...');
            die;
        }

        if (intval(Tools::getValue('europe')))
            $this->europe = 1;
        else
            $this->europe = 0;


        //  Check Access Tokens
        //
        $tokens = Tools::getValue('amazon_token');

        if (!Amazon_Tools::checkToken($tokens, $id_lang, $this->europe))
            die('Wrong Token');

        // Init
        //
        $amazon = Amazon_Tools::selectPlatforms($id_lang, $this->_debug);

        if ($this->_debug)
            echo nl2br(print_r($amazon['auth'], true) . print_r($amazon['params'], true) . print_r($amazon['platforms'], true));

        if (!($this->_amazonApi = new Service($amazon['auth'], $amazon['params'], $amazon['platforms'], $this->_debug)))
        {
            echo $this->l('Unable to login') . $cr;
            die;
        }


        // Listing Orders
        //
        $orders = $this->_amazonApi->GetUnshippedOrdersList($date1, $date2, $status);

        if (!$orders || !count($orders))
        {
            echo '<h1>' . $this->l('No pending order for the selected period') . '</h1>' . $cr;
            die;
        }


        echo '<form name="orders" id="orders"><table width="100%" class="table">';
        echo '<input type="hidden" name="amazon_lang" value="' . $id_lang . '" />';
        echo '<thead>
            <th><input type="checkbox" name="checkme" value="%s" /></th>' .
        '<th></th>' .
        '<th>' . $this->l('Date') . '</th>' .
        '<th>' . $this->l('ID') . '</th>' .
        '<th>' . $this->l('Status') . '</th>' .
        '<th>' . $this->l('Customer') . '</th>' .
        '<th>' . $this->l('Shipment') . '</th>' .
        '<th>' . $this->l('FF/by') . '</th>' .
        '<th>' . $this->l('Quantity') . '</th>' .
        '<th>' . $this->l('Total') . '</th>' .
        '</thead>';


        $irow = 0;

        $orderCheck = new Amazon_Order();

        // Identify the order's country
        //
        $marketPlaceIds = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MARKETPLACE_ID')));
        $marketPlace2IsoId = array_flip($marketPlaceIds);

        foreach ($orders as $key => $order)
        {
            // Langue de la Commande
            //
            if (isset($this->europe) && isset($marketPlace2IsoId))
                $id_lang = $marketPlace2IsoId[$order->MarketPlaceId];
            else
                $id_lang = $id_lang;

            $currency = (int) Currency::getIdByIsoCode(strval($order->OrderTotalCurrency));

            $total = 0;
            $shipping = 0;
            $quantities = 0;
            //$fees  = $order->fees	;
            $fees = 0;

            $total = floatval($order->OrderTotalAmount);
            $quantities = intval($order->NumberOfItemsUnshipped);
            $retrieved = $orderCheck->checkByMpId($order->AmazonOrderId);

            if ($retrieved)
                $disabled = ' disabled="disabled" ';
            else
                $disabled = '';

            $url = '?tab=AdminOrders&id_order=' . $retrieved . '&vieworder&token=' . $tokenOrders;

            $order_link = $retrieved ? ('<a href="' . $url . '" title="" target="_blank" >' . $order->AmazonOrderId . '</a>') : $order->AmazonOrderId;

            if ($retrieved)
                $disabled = ' disabled="disabled" ';
            else
                $disabled = '';

            if (isset($id_lang))
                $flag = '<img src="../img/l/' . $id_lang . '.jpg" alt="" />';
            else
                $flag = '';

            printf('
                <tr class="' . ($irow++ % 2 ? 'alt_row' : '') . '">
                <td>
                    <input type="checkbox" id="order_' . $irow . '" ' . $disabled . ' name="order_id[]" value="%s" />
                </td>
                <!-- flag -->
                <td>%s</td>
                <!-- created_at -->
                <td>%s</td>
                <!-- order_id -->
                <td class="order_id">%s</td>
                <!-- state -->
                <td>%s</td>
                <!-- customer -->
                <td style="width:200px;">%s</td>
                <!-- service -->
                <td align="right">%s</td>
                <!-- fba -->
                <td align="right">%s</td>
                <!-- quantities -->
                <td align="right">%s</td>
                <!-- total -->
                <td align="right">%s</td>
                </tr>
                ', $order->AmazonOrderId, $flag, Tools::displayDate(date('Y-m-d H:i:s', strtotime($order->PurchaseDate)), $id_lang, true), $order_link, $order->OrderStatus, $order->Address->Name, $order->ShipServiceLevel, $order->FulfillmentChannel, intval($order->NumberOfItemsUnshipped) + intval($order->NumberOfItemsShipped), Tools::displayPrice($total, $currency)
            );
        }

        echo '</table></form>
      <br />

      <input type="button" class="button" id="retrieve" value="' . $this->l('Retrieve Selected Orders') . '" />
      ';
    }

}

$amazonOrders = new AmazonListOrder();
$amazonOrders->Dispatch('list');
?>