<?php


@require_once(dirname(__FILE__).'/../../../config/config.inc.php');
@require_once(dirname(__FILE__).'/../../../init.php');

require_once(dirname(__FILE__).'/../amazon.php');

require_once(dirname(__FILE__).'/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.order.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.product.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.webservice.class.php');

define('AMAZON_TRADER', Amazon::configurationGet('AMAZON_TRADER') ? true : false ) ;

if ( defined('AMAZON_TRADER') && AMAZON_TRADER )
{
  require_once(dirname(__FILE__).'/../classes/amazon.strategies.class.php');
  require_once(dirname(__FILE__).'/../classes/amazon.webservice-bid.class.php');
}

class Amazon_ProductOptionsJSON extends Module
{
  const cr =  "<br />\n" ;
  private $module   = 'amazon';
  public $name   = 'amazon';

  private $auth;
  private $debug;
  private $region;

  private $_amazon ;

  function __construct()
  {
    global $cookie, $_MODULE;

    $langFile = _PS_MODULE_DIR_.$this->module.'/'.Language::getIsoById(intval($cookie->id_lang)).'.php';
    $mySelf   = basename(__FILE__, '.php') ;

    if(file_exists($langFile))
    {
      require_once $langFile;
      foreach($_MODULE  as $key=>$value)
          $_MODULE[str_replace($mySelf . '_' , $this->name . '_', $key)] = $value;
    }

    $debugMode = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG'))) ;
    $this->debug = intval($debugMode) ? true : false ;

    $this->_amazon = new Amazon() ;

    parent::__construct() ;
  }
  public function l($string, $specific = false, $id_lang = null)
  {
    return( $this->_amazon->l($string, $specific = false, $id_lang = null) ) ;
  }

  public function InitAmazon($id_lang)
  {
    // Amazon Europe overidding
    //
    $amazonEurope      = unserialize(base64_decode(Amazon::configurationGet('AMAZON_EUROPE'))) ;
    $marketPlaceMaster = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MASTER'))) ;
    $marketPlaceRegion = unserialize(base64_decode(Amazon::configurationGet('AMAZON_REGION'))) ;
    $marketPlaceIds    = unserialize( base64_decode(Amazon::configurationGet('AMAZON_MARKETPLACE_ID')) ) ;

    // Acc�s API Amazon
    //
    $merchantIds        = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MERCHANT_ID'))) ;
    $marketPlaceIds     = unserialize(base64_decode(Amazon::configurationGet('AMAZON_MARKETPLACE_ID'))) ;
    $awsKeyIds          = unserialize(base64_decode(Amazon::configurationGet('AMAZON_AWS_KEY_ID'))) ;
    $awsSecretKeys      = unserialize(base64_decode(Amazon::configurationGet('AMAZON_SECRET_KEY'))) ;

    $amazonCurrency = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CURRENCY'))) ;

    // Currencies
    //
    $currencies         = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CURRENCY'))) ;

    $this->platforms = array() ;

    if ( intval($amazonEurope) )
    {
      foreach($marketPlaceRegion as $language_id => $region)
      {
        // Identify the Master Marketplace
        //
        if ( $marketPlaceMaster == $marketPlaceRegion[$language_id] )
        {
          $mp_id_lang = $language_id ;   // Default Platform - Language

          $merchantId         = trim($merchantIds[$language_id]) ;
          $marketPlaceId      = trim($marketPlaceIds[$language_id]) ;
          $awsKeyId           = trim($awsKeyIds[$language_id]) ;
          $awsSecretKey       = trim($awsSecretKeys[$language_id]) ;

          $this->auth = array(
               'MerchantID'     => $merchantId,
               'MarketplaceID'  => $marketPlaceId,
               'AWSAccessKeyID' => $awsKeyId,
               'SecretKey'      => $awsSecretKey
          );
        }
        $this->params[ $currencies[$language_id] ] = array() ;
        $this->params[ $currencies[$language_id] ]['Currency'] = $amazonCurrency[$language_id] ;
        $this->params[ $currencies[$language_id] ]['Country'] = $marketPlaceMaster ;

        $this->platforms[ $currencies[$language_id] ][] = $marketPlaceIds[$language_id] ;
      }

      if ( ! isset($mp_id_lang) )
        die($this->l('Amazon Europe : missing Master Platform configuration')) ;
    }
    else
    {
      $merchantId         = trim($merchantIds[$id_lang]) ;
      $marketPlaceId      = trim($marketPlaceIds[$id_lang]) ;
      $awsKeyId           = trim($awsKeyIds[$id_lang]) ;
      $awsSecretKey       = trim($awsSecretKeys[$id_lang]) ;

      $this->auth = array(
           'MerchantID'     => $merchantId,
           'MarketplaceID'  => $marketPlaceId,
           'AWSAccessKeyID' => $awsKeyId,
           'SecretKey'      => $awsSecretKey
      ) ;
      $this->platforms = array() ;
      $this->platforms[ $currencies[$id_lang] ][] = $marketPlaceIds[$id_lang] ;

      $this->params[ $currencies[$id_lang] ]['Currency'] = $amazonCurrency[$id_lang] ;
      $this->params[ $currencies[$id_lang] ]['Country'] = $marketPlaceRegion[$id_lang] ;
    }
    $this->region = $this->params[ $currencies[$id_lang] ] ;

  }

  public function doIt()
  {
      $callback     = Tools::getValue('callback') ;
      $langs        = Tools::getValue('amazon_option_lang') ;
      $id_product   = Tools::getValue('id_product') ;
      $id_category  = Tools::getValue('id_category_default') ;
      $id_manufacturer = Tools::getValue('id_manufacturer') ;

      switch( Tools::getValue('action') )
      {
          case 'set' : // Set Product Opton
              $pass = true ;

              foreach($langs as $key => $val)
              {

                  $id_lang      = intval($val) ;
                  $disable      = Tools::getValue('amz-disable-' . intval($id_lang)) ? 1 : 0 ;
                  $force        = Tools::getValue('amz-force-' . intval($id_lang))  ? 1 : 0 ;
                  $price        = Tools::getValue('amz-price-' . intval($id_lang));
                  $text         = Tools::getValue('amz-text-' . intval($id_lang));
                  $nopexport    = Tools::getValue('amz-nopexport-' . intval($id_lang));
                  $noqexport    = Tools::getValue('amz-noqexport-' . intval($id_lang));
                  $fba          = Tools::getValue('amz-fba-' . intval($id_lang));
                  $latency      = Tools::getValue('amz-latency-' . intval($id_lang));
                  $asin1        = Tools::getValue('amz-asin-' . intval($id_lang));
                  $asin2        = Tools::getValue('amz-asin-2-' . intval($id_lang));
                  $asin3        = Tools::getValue('amz-asin-3-' . intval($id_lang));
                  $shipping     = Tools::getValue('amz-shipping-' . intval($id_lang));
                  $shipping_type= Tools::getValue('amz-overridetype-' . intval($id_lang));

                  $options      = array('force' => $force, 'nopexport' => $nopexport, 'noqexport' => $noqexport, 'fba' => $fba, 'latency' => $latency, 'disable' => $disable, 'price' => $price, 'asin1' => $asin1, 'asin2' => $asin2, 'asin3' => $asin3, 'text' => $text, 'shipping' => $shipping, 'shipping_type' => $shipping_type) ;

                  if ( ! Amazon_Product::setProductOptions($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-text-cat' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $text   = Tools::getValue('amz-text-' . intval($id_lang));

                  $options = array('text' => $text) ;

                  if ( ! Amazon_Product::propagateProductOptionsText($id_product, $id_lang, $id_category, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-text-shop' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $text    = Tools::getValue('amz-text-' . intval($id_lang));

                  $options = array('text' => $text) ;

                  if ( ! Amazon_Product::propagateToShopProductOptionsText($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-latency-cat' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $latency   = Tools::getValue('amz-latency-' . intval($id_lang));

                  $options = array('latency' => $latency) ;

                  if ( ! Amazon_Product::propagateProductOptionsLatency($id_product, $id_lang, $id_category, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-latency-shop' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $latency    = Tools::getValue('amz-latency-' . intval($id_lang));

                  $options = array('latency' => $latency) ;

                  if ( ! Amazon_Product::propagateToShopProductOptionsLatency($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;


          case 'propagate-shipping-cat' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $shipping   = Tools::getValue('amz-shipping-' . intval($id_lang));
                  $shipping_type   = intval(Tools::getValue('amz-overridetype-' . intval($id_lang)));

                  $options = array('shipping' => is_numeric($shipping) ? floatval($shipping) : NULL, 'shipping_type' => $shipping_type) ;

                  if ( ! Amazon_Product::propagateProductOptionsShipping($id_product, $id_lang, $id_category, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-shipping-shop' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $shipping   = Tools::getValue('amz-shipping-' . intval($id_lang));
                  $shipping_type   = intval(Tools::getValue('amz-overridetype-' . intval($id_lang)));

                  $options = array('shipping' => is_numeric($shipping) ? floatval($shipping) : NULL, 'shipping_type' => $shipping_type) ;

                  if ( ! Amazon_Product::propagateToShopProductOptionsShipping($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;


          case 'propagate-disable-cat' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $disable = intval(Tools::getValue('amz-disable-' . intval($id_lang))) ;

                  $options = array('disable' => $disable) ;

                  if ( ! Amazon_Product::propagateProductOptionsDisable($id_product, $id_lang, $id_category, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-disable-shop' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $disable = intval(Tools::getValue('amz-disable-' . intval($id_lang)));

                  $options = array('disable' => $disable) ;

                  if ( ! Amazon_Product::propagateToShopProductOptionsDisable($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-disable-manufacturer' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $disable = intval(Tools::getValue('amz-disable-' . intval($id_lang)));

                  $options = array('disable' => $disable) ;

                  if ( ! Amazon_Product::propagateToManufacturerProductOptionsDisable($id_product, $id_lang, $id_manufacturer, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

         // Force
         //

          case 'propagate-force-cat' : // Propagate product option force
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $force   = intval(Tools::getValue('amz-force-' . intval($id_lang)));

                  $options = array('force' => $force) ;

                  if ( ! Amazon_Product::propagateProductOptionsForce($id_product, $id_lang, $id_category, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-force-shop' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $force   = intval(Tools::getValue('amz-force-' . intval($id_lang)));

                  $options = array('force' => $force) ;

                  if ( ! Amazon_Product::propagateToShopProductOptionsForce($id_product, $id_lang, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'propagate-force-manufacturer' : // Propagate product option text
              $pass = true ;

              foreach($langs as $key => $val)
              {
                  $id_lang = intval($val) ;
                  $force   = intval(Tools::getValue('amz-force-' . intval($id_lang)));

                  $options = array('force' => $force) ;

                  if ( ! Amazon_Product::propagateToManufacturerProductOptionsForce($id_product, $id_lang, $id_manufacturer, $options) )
                    $pass = false ;
              }
              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;


          case 'reinit-strategies' : // Propagate product option text
              if ( defined('AMAZON_TRADER') && AMAZON_TRADER )
                $pass = Amazon_Strategy::reInit() ;

              if ( $pass )
                    echo $this->l('Parameters successfully saved') ;
              else  echo $this->l('Unable to save parameters...') ;

              break ;

          case 'ean2asin' :
                $id_lang = Tools::getValue('id_lang') ;
                $pass = true ;
                $errors = null ;
                $output = null ;
                $asin = '' ;

                ob_start();
                $this->InitAmazon($id_lang) ;

                if ( ! ($amazonApi = new Service($this->auth, $this->region, false, $this->debug)) )
                {
                  echo $n->l('Unable to login') . $cr ;
                  die;
                }

                if ( $lookup = Tools::getValue('ean13') )
                {
                   $result = $amazonApi->getASIN($lookup) ;

                   if ( isset($result[0]) && ! empty($result[0]) )
                   {
                    echo $this->l('ASIN sucessfully fetched') . self::cr ;
                    $asin = $result[0] ;
                   }

                }
                elseif ( $lookup = intval(Tools::getValue('upc')) )
                {
                   $result = $amazonApi->getASIN($lookup) ;

                   if ( isset($result[0]) && ! empty($result[0]) )
                   {
                    echo $this->l('ASIN sucessfully fetched') . self::cr ;
                    $asin = $result[0] ;
                   }
                }

                if ( ! $asin )
                {
                  echo sprintf('%s %s %s', $this->l('Unable to fetch ASIN for'), $lookup, self::cr) ;
                  $pass = false ;
                }


                $output = ob_get_clean() ;


                $json = json_encode( array('output' => $output, 'error' => ! $pass, 'asin' => $asin) ) ;

                echo   Tools::getValue('callback') . '(' . $json . ')';
                die ;
              break ;

          default :
              break ;

      }
    }
}
$apoJSON = new Amazon_ProductOptionsJSON() ;
$apoJSON->doIt() ;
?>