<?php

@require_once(dirname(__FILE__) . '/../../../config/config.inc.php');
@require_once(dirname(__FILE__) . '/../../../init.php');

require_once(dirname(__FILE__) . '/../amazon.php');

require_once(dirname(__FILE__) . '/../classes/amazon.product.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__) . '/../classes/amazon.webservice.class.php');

require_once(_PS_MODULE_DIR_ . 'amazon/validate/Node.php'); 
require_once(_PS_MODULE_DIR_ . 'amazon/validate/XmlDataType.php'); 
require_once(_PS_MODULE_DIR_ . 'amazon/validate/XmlRestriction.php'); 
require_once(_PS_MODULE_DIR_ . 'amazon/validate/AmazonXSD.php'); //gets code for Amazon XML Schemas
require_once(_PS_MODULE_DIR_ . 'amazon/validate/AmazonValidator.php'); 
require_once(_PS_MODULE_DIR_ . 'amazon/validate/Tools.php'); 

class Amazon_ExportProducts extends Module
{

    private $_auth = array();
    private $_param = array();
    private $_amazonApi = array();
    private $_debug = false;
    private $module = 'amazon';
    public $name = 'amazon';
    private $_amazon;

    public $id_lang ;
    
    function __construct()
    {
        global $cookie, $_MODULE, $_LANGADM;

        $this->id_lang = (int)Tools::getValue('id_lang');
        
        if ( ! $this->id_lang ) 
            $this->id_lang = Configuration::get('PS_LANG_DEFAULT') ;
        
        $langFile = _PS_MODULE_DIR_ . $this->module . '/' . Language::getIsoById(intval($this->id_lang)) . '.php';
        $mySelf = basename(__FILE__, '.php');

        if (file_exists($langFile))
        {
            require_once $langFile;
            if ( $_MODULE )
            foreach ($_MODULE as $key => $value)
                $_MODULE[str_replace($mySelf . '_', $this->name . '_', $key)] = $value;
        }
        $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/';
        $this->images = $module . 'images/';

        $this->_amazon = new Amazon();

        parent::__construct();
    }

    public function l($string, $specific = false, $id_lang = null)
    {
        return( $this->_amazon->l($string, $specific = false, $id_lang = null) );
    }

    function DoIt()
    {
        global $cookie, $cart, $argv;

        $cronToken = false;

        //
        // Cron Mode
        //
        if (Tools::getValue('cron'))
        {
            $lang = Tools::getValue('lang');
            $tokens = Tools::getValue('cron_token');

            $cr = "<br />\n"; // carriage return

            echo $this->l("Starting Update in WS API/Cron Mode") . ' - ' . date('Y-m-d H:i:s') . $cr;

            // Regions
            //
            $marketPlaceRegion = unserialize(base64_decode(Amazon::configurationGet('AMAZON_REGION')));    
            $marketLang2Region = array_flip($marketPlaceRegion) ;

            if ( ! isset($marketLang2Region[$lang]) || ! $marketLang2Region[$lang] )
                die($this->l('No selected language, nothing to do...')) ;    

            $id_lang = $marketLang2Region[$lang] ;

            if (intval(Tools::getValue('europe')))
                $europe = 1;
            else
                $europe = 0;

            $importDate = Amazon::configurationGet('AMAZON_LAST_CRON_UPDATED_' . $id_lang);

            $cronMode = true;
            $sendToAmazon = true;
        }
        //
        // Web Mode
        //
        else
        {
            $cr = "<br />\n"; // carriage return

            $importDate = Tools::getValue('importDate');

            echo $this->l("Starting Update in WS API/Web Mode") . ' - ' . date('Y-m-d H:i:s') . ' - ' . $importDate . $cr;

            $id_lang = intval(Tools::getValue('amazon_lang'));

            if (intval(Tools::getValue('europe')))
                $europe = 1;
            else
                $europe = 0;

            $tokens = Tools::getValue('amazon_token');

            $lastUpdated = Amazon::configurationGet('AMAZON_LAST_UPDATED_' . $id_lang);

            if (!intval($id_lang))
            {
                echo $this->l('No selected language, nothing to do...');
                die;
            }
            $cronMode = false;
            $sendToAmazon = false;
        }

        // Global Parameters
        //
        $params = new Params($id_lang);
        $params->europe = $europe;
        $params->cronMode = $cronMode;
        $params->sendToAmazon = $sendToAmazon;
        $params->sendProductDescription = intval(Tools::getValue('synch-descriptions')) ? true : false;
        $params->relationShipsOnly = intval(Tools::getValue('relations-only')) ? true : false;
        $params->xmlOnly = intval(Tools::getValue('xml-only')) ? true : false;
        $params->action = Tools::getValue('action');
        $params->sendActives = Tools::getValue('active_only');

        if ( $params->relationShipsOnly )   $params->create = true ;
        
        if ( $params->xmlOnly )   
        {
            $params->sendProductDescription = true ;
            $params->create = true ;
        }

        // Attributes
        // 
        $Attributes = Attribute::getAttributes($params->id_lang);

        // Security Check
        //
        if (!Amazon_Tools::checkToken($tokens, $id_lang, $params->europe))
            die('Wrong Token');

        // Init
        //
        $params->platform = Amazon_Tools::selectPlatform($params->id_lang, $params->debug);

        $params->toCurrency = new Currency(Currency::getIdByIsoCode($params->platform['params']['Currency']));
        $params->fromCurrency = new Currency((int) (Amazon::configurationGet('PS_CURRENCY_DEFAULT')));

        if ($params->debug)
            echo nl2br(print_r($params->platform['auth'], true) . print_r($params->platform['params'], true) . print_r($params->platform['platforms'], true));

        $params->marketplaceID = $params->platform['auth']['MarketplaceID'] ;
        $params->fullfillmentCenterId = Amazon_Tools::marketplaceIdToFullfillmentCenterId($params->platform['auth']['MarketplaceID']) ;
        
        if (!($this->_amazonApi = new Service($params->platform['auth'], $params->platform['params'], null, $params->debug)))
        {
            echo $this->l('Unable to login') . $cr;
            die;
        }

        // Amazon Throtlling Management
        // 
        $this->_amazonApi->setTimerReadHandler('Configuration', 'get', 'AMAZON_API_TIMER');
        $this->_amazonApi->setTimerWriteHandler('Configuration', 'updateValue', 'AMAZON_API_TIMER');


        if (defined('AMAZON_TRADER') && AMAZON_TRADER)
            require_once(dirname(__FILE__) . '/../classes/amazon.strategies.class.php');

        if ($params->bruteForce)
            echo "Operating in brute force mode !<br />\n";

        if (!$params->categories || !count($params->categories))
        {
            echo $this->l("Categories Settings not saved yet, nothing to do...") . $cr;
            die;
        }

        $params->syncBothMode = false;

        switch ($params->synchField)
        {
            case 'ean13' :
                $ps_code = 'ean13';
                $az_code = 'EAN';
                break;
            case 'upc' :
                $ps_code = 'upc';
                $az_code = 'UPC';
                break;
            case 'reference' :
                $ps_code = 'reference';
                $az_code = 'SKU';
                break;
            case 'both' :
                $params->syncBothMode = true;
                break;
        }
        
        if ($params->debug)
            if ($params->syncBothMode == true)
                printf($this->l('Synchronizing field are Prestashop: UPC/EAN (both)') . $cr);
            else
                printf($this->l('Synchronizing field are Prestashop: %s / Amazon: %s') . $cr, $ps_code, $az_code);

        // Synch with ASIN
        //
        if ($params->debug && $params->asinHasPriority)
            printf($this->l('But ASIN has the priority !') . $cr);

        $productsUpdate = array();
        $productsDelete = array();
        $u = 0;
        $d = 0;
        $duplicate = 0;
        $skipped = 0;
        $history = array();
        $Relationship = array();
        $parsedXSD = array() ;
        
        switch ($params->action)
        {
            case 'update' :
                $params->sendToAmazon = true;
                break;
            case 'export' :
                $params->sendToAmazon = true;
                $params->bruteForce = true;
                break;
            case 'verify' :
                $params->sendToAmazon = false;
                $params->bruteForce = true;            
                break;
            default :
                $params->sendToAmazon = false;
                break;
        }

        if ( $params->create )
        {
            if ( $params->debug)
                printf('%s:%d - ' . $this->l('Starting in creation mode'). $cr, basename(__FILE__), __LINE__);                
            
            $this->_amazonApi->setOperationMode(Service::OPERATIONS_CREATE); 
            
            // Force Send Title & Description on Creation
            // 
            $params->sendProductDescription = true ;
        }
        
        //  Products Update
        //
        $i = 0;

        // Envoi global ou mode brute 
        //
        if ($params->bruteForce)
        {
            // Clear Action Table 
            //
            if ( ! $params->create )
                Amazon_Product::marketplaceActionReset($params->id_lang);

            $p = Amazon_Product::marketplaceGetAllProducts($params->id_lang, $params->sendActives, $params->dateSince);
        }
        else
        {
            $p = Amazon_Product::marketplaceActionList($params->id_lang);
        }

        if ($p)
        foreach ($p as $key => $val)
        {
            // Action for this Product
            //
            $action = isset($val['action']) ? $val['action'] : Amazon::Add;

            // Period
            if (!$params->updateDateStart)
                $params->updateDateStart = isset($val['date_start']) ? $val['date_start'] : null;
            if (isset($val['date_stop']) && $val['date_stop'] > $params->updateDateStop)
                $params->updateDateStop = $val['date_stop'];

            $id_product = intval($val['id_product']);

            // Product
            //
            $details = new Product($id_product, false, $params->id_lang);

            // For product removal
            //
            if (!Validate::isLoadedObject($details))
            {    
                $details = new Product();
                $details->id = $id_product ;
            }

            // Both Synch Mode (EAN/UPC)
            //
            if ($params->syncBothMode)
            {
                $az_code = 'EAN';
                $ps_code = 'ean13';   
                
                if (isset($details->ean13) && !empty($details->ean13))
                {
                    $az_code = 'EAN';
                    $ps_code = 'ean13';
                }
                elseif (isset($details->upc) && !empty($details->upc))
                {
                    $az_code = 'UPC';
                    $ps_code = 'upc';
                }
            }

            // Profiles
            //
            $productData = NULL;
            $manufacturer = NULL;

            $profile_name = null;
            $profile_id = null;

            $variant = false ;

            $p_product_type = null ;
            $p_name = null ;
            $p_universe = null ;
            $p_formula = null ;
            $p_product_type = null ;
            $p_browsenode = null ;
            $p_field_size = null ;
            $p_field_color = null ;
            $p_parameters = null ;
            $p_category = null ;
            $p_extra = null ;
            $p_code_exemption = null ;
            $skip = false ;

            $parsedXSD = array() ;

            if ( isset($params->profile2category[$details->id_category_default]) )
            {
                if ( in_array($details->id_category_default, $params->categories) )
                {
                    $profile_name = $params->profile2category[$details->id_category_default];
                    $profile_id = false;

                    if ( $params->profile['name'] )
                        foreach ($params->profile['name'] as $profile_id => $profile)
                            if ($profile == $profile_name)
                                break;

                    if ($profile_id !== false && ! empty($profile_name) )
                    {
                        if ($params->debug)
                            printf('Using profile [%s] ID: %s' . $cr, $profile_name, $profile_id);
                    }
                    elseif ( $params->create ) $skip = true ;    
                }
                else
                {
                    if ($params->debug)
                        printf('Profil is not in profiles list [%s] id: %s' . $cr, $profile_name, $profile_id);                        
                    if ( $params->create ) $skip = true ;
                }
                                
            }
            elseif ( $params->create ) 
                $skip = true ;

            if ( $profile_name && is_numeric($profile_id) )
            {
                if ($params->debug)
                    printf('Using profile : %s' . $cr, $profile_name);
                
                $p_product_type = $params->profile['product_type'][$profile_id];
                $p_name = $params->profile['name'][$profile_id];
                $p_universe = $params->profile['universe'][$profile_id];
                $p_formula = $params->profile['formula'][$profile_id];
                $p_product_type = $params->profile['product_type'][$profile_id];
                $p_browsenode = isset($params->profile['browsenode_' . $params->id_lang][$profile_id]) ? $params->profile['browsenode_' . $params->id_lang][$profile_id] : null ;
                $p_field_size = $params->profile['field_size'][$profile_id];
                $p_field_color = $params->profile['field_color'][$profile_id];
                $p_parameters = $params->profile['parameters'][$profile_id];
                $p_category = $params->profile['category'][$profile_id];
                $p_code_exemption = $params->profile['code_exemption'][$profile_id] ? true : false ;
            }    

            if ($params->create && $profile_name)
            {                
                // Load XSD into an array - will be used to parse sort the XML as defined in the XSD
                //
                $xsdStructure = array();

                if ( $p_category && ! isset($parsedXSD[$p_category]) )
                {
                    try
                    {
                        $productFactory = new AmazonXSD($p_category.'.xsd');
                        $productInstance = $productFactory->getInstance();
                    }
                    catch(Exception $e)
                    {
                        die($this->l('Amazon XSD Exception :') . $e->getMessage()) ;
                    }  
                    
                    $p_parameters['xsd'] = $parsedXSD[$p_category] = $productFactory->getInstanceElementsArray($xsdStructure, $productInstance->ProductData); 

                    if ($params->debug)
                        printf('Creation mode: parsing XSD for category : %s' . $cr, $p_category);

                }
                elseif ( $p_category && isset($parsedXSD[$p_category]) )
                {
                    $p_parameters['xsd'] = $parsedXSD[$p_category] ;
                    
                    if ($params->debug)
                        printf('Creation mode: using parsed XSD for category : %s' . $cr, $p_category);                    
                }
                else
                {
                    printf('Unable to parse XSD definitions for profile %s' . $cr, $profile_name) ;
                    continue ;
                }

                if ( isset($params->profile['extra'][$profile_name]) )
                     $p_extra = $params->profile['extra'][$profile_name] ;
                else $p_extra = null ;
                
                if (!$p_product_type)
                {
                    if ($params->debug)
                        printf('Skipped: product has no product type - %s' . $cr, $id_product);                    
                    $skip = true ;
                }
            }
            elseif ($params->create) $skip = true ;
            
            if ( $skip ) // mode creation - we filter the product which not have a profile
            {
                if ($params->debug)
                    printf('Skipped: product has no or wrong profile - %s' . $cr, $id_product);
                continue ;
            }
            
            $manufacturer = null ;
            
            // Fields required for product sheet creation
            //
            if ( $details->id_manufacturer )
                $manufacturer = Manufacturer::getNameById($details->id_manufacturer);

            // Product Combinations
            //
            if (version_compare(_PS_VERSION_, '1.5', '<'))
                $combinations = $details->getAttributeCombinaisons($params->id_lang);
            else
                $combinations = $details->getAttributeCombinations($params->id_lang);

            if (isset($val['sku']) && !empty($val['sku']))
                $reference = $val['sku'];
            else
                $reference = $details->reference;

            if (!isset($combinations) or !is_array($combinations) or empty($combinations))
                $combinations = array(0 => array(
                        'reference'            => $reference,
                        'price'                => 0,
                        'quantity'             => isset($details->quantity) ? $details->quantity : 0,
                        $ps_code               => isset($details->$ps_code) &&  $details->$ps_code ? $details->$ps_code : '',
                        'id_product_attribute' => 0,
                        'default_on'           => 0
                        ));

            $group_details = array();
            $color = $size = null;
            
            asort($combinations) ;
            
            // Grouping Combinations
            //
            foreach ($combinations as $combination)
            {
                $id_product_attribute = $combination['id_product_attribute'];

                if (! isset($group_details[$id_product_attribute]))
                    $group_details[$id_product_attribute] = array();
                $group_details[$id_product_attribute]['reference'] = $combination['reference'];

                // Attributes
                //
                $id_attribute_group = isset($combination['id_attribute_group']) ? $combination['id_attribute_group'] : null;
                $id_attribute = isset($combination['id_attribute']) ? $combination['id_attribute'] : null;

                if ( !isset($group_details[$id_product_attribute]['attribute_name']) )
                    $group_details[$id_product_attribute]['attribute_name'] = null ;
                
                if ( isset($combination['attribute_name']) && ! empty($combination['attribute_name']) )
                    $group_details[$id_product_attribute]['attribute_name'] .= sprintf('%s, ', $combination['attribute_name']) ;

                $group_details[$id_product_attribute]['quantity'] = $combination['quantity'];
                $group_details[$id_product_attribute]['default_on'] = $combination['default_on'];


                // Synch Field (EAN, UPC, SKU ...)
                //
                $group_details[$id_product_attribute][$ps_code] = $combination[$ps_code];

                // Attributes Values (Color, Size)...
                //
                if (isset($params->mapping_right[$params->id_lang][$id_attribute_group]) && isset($params->mapping_right[$params->id_lang][$id_attribute_group][$id_attribute]) && $params->create)
                {
                    if ($id_attribute_group == $p_field_color)
                    {
                        foreach ($Attributes as $attribute)
                            if ($attribute['id_attribute_group'] == $id_attribute_group && $attribute['id_attribute'] == $id_attribute)
                                $color = $attribute['name'];

                        $group_details[$id_product_attribute]['ColorMap'] = trim($params->mapping_right[$params->id_lang][$id_attribute_group][$id_attribute]) ;
                        $group_details[$id_product_attribute]['Color'] = trim($color) ;
                    }
                    if ($id_attribute_group == $p_field_size)
                    {
                        foreach ($Attributes as $attribute)
                            if ($attribute['id_attribute_group'] == $id_attribute_group && $attribute['id_attribute'] == $id_attribute)
                                $size = $attribute['name'];

                        $group_details[$id_product_attribute]['SizeMap'] = trim($params->mapping_right[$params->id_lang][$id_attribute_group][$id_attribute]) ;
                        $group_details[$id_product_attribute]['Size'] = trim($size) ;
                    }
                }
            }
            $masterProduct = 0;
            $productIndex = 0 ;
            
            $details->name_attributes = null ;
                        
            // Export Combinations or Products Alone
            //
            foreach ($group_details as $id_product_attribute => $combination)
            {
                if ($params->debug)
                    echo nl2br(print_r($combination, true));

                if ( $id_product_attribute && ! $productIndex )
                        $masterProduct = 1 ;
                else    $masterProduct = 0 ;
                
                $productIndex++ ;

                if ( ! $params->cronMode )  echo "<br />" ;
                
                if ( isset($combination['reference']) && ! empty($combination['reference']) && $az_code && $combination[$ps_code] )
                    printf('%s - ID: %d/%d - SKU: %s - %s: %s' . $cr, $details->name, $id_product, $id_product_attribute, $combination['reference'], $az_code, $combination[$ps_code]) ;
                
                if ( isset($combination['reference']) && ! empty($combination['reference']) && isset($history[ $combination['reference'] ]) )
                {
                    echo '<span style="color:brown;font-weight:bold;">' ;
                    printf($this->l('Duplicate entry for product %s(%d/%d) - This is not allowed by Amazon - Skipping product') . $cr, $combination['reference'], $id_product, $id_product_attribute);
                    $productIndex-- ;
                    $skipped++;
                    echo '</span>' ;
                    continue ;
                }
                $history[ $combination['reference'] ] = true ;
           
                if (empty($combination['reference']) && $ps_code != 'reference')
                {
                    if ( $params->debug )
                        printf( '%s:%s - ', basename(__FILE__), __LINE__);
                    printf( $this->l('Skipping Product ID: %d/%d - Has No Reference') . $cr, $details->id, $id_product_attribute);
                    
                    $productIndex-- ;
                    $skipped++;
                    continue;
                }

                if (empty($combination[$ps_code]) && !$params->asinOverride && $action != Amazon::Remove && $ps_code != 'reference' && ! $p_code_exemption )
                {
                    if ( $params->debug )
                        printf( '%s:%s - ', basename(__FILE__), __LINE__);
                    printf( $this->l('Skipping Product ID: %d/%d - Has No %s') . $cr, $details->id, $id_product_attribute, strtoupper($ps_code)) ;
                    
                    $productIndex-- ;
                    $skipped++;
                    continue;
                }    
                
                if ( $params->synchField != 'reference' && ! $p_code_exemption )
                {
                    $EAN_UPC = isset($combination[$ps_code]) && ! empty($combination[$ps_code]) ? $combination[$ps_code] :
                               (isset($details->$ps_code) && ! empty($details->$ps_code) ? $details->$ps_code : null) ;
                    
                    if ( ! $EAN_UPC )
                    {
                        if ($params->debug)
                            printf('Skipping product %s which has no %s' . $cr, $id_product, $ps_code);
                        $productIndex-- ;
                        $skipped++;
                        continue ;
                    }

                    // Check if the code is a "reserved for local use" code.
                    //
                    if ( Amazon_Tools::EAN_UPC_isPrivate($EAN_UPC) )
                    {
                        if ( ! $params->cronMode ) echo '<span style="color:brown;font-weight:bold;">' ;
                        printf($this->l('Inconsistency for product %s(%d/%d) - Product code %s(%s) is reserved for local use - Skipping product') . $cr, $combination['reference'], $id_product, $id_product_attribute, $ps_code, $EAN_UPC);
                        if ( ! $params->cronMode ) echo '</span>' ;
                        $productIndex-- ;
                        $skipped++ ;
                        continue ;                        
                    }
                    
                    // Check EAN/UPC consistency
                    //
                    if ( ! Amazon_Tools::EAN_UPC_Check($EAN_UPC) )
                    {
                        if ( ! $params->cronMode ) echo '<span style="color:brown;font-weight:bold;">' ;
                        printf($this->l('Inconsistency for product %s(%d/%d) - Product code %s(%s) seems to be wrong - Skipping product') . $cr, $combination['reference'], $id_product, $id_product_attribute, $ps_code, $EAN_UPC);
                        if ( ! $params->cronMode ) echo '</span>' ;
                        $productIndex-- ;
                        $skipped++ ;
                        continue ;                        
                    }
                           
                }

                if (isset($combination['attribute_name']) )
                    $details->name_attributes = $combination['attribute_name'] ;
                    
                if (version_compare(_PS_VERSION_, '1.5', '>='))
                    $quantity = Product::getRealQuantity($details->id, $id_product_attribute ? $id_product_attribute : null, $params->id_warehouse, $params->id_shop);
                else
                    $quantity = Product::getQuantity($details->id, $id_product_attribute ? $id_product_attribute : null);

                $productData = null ;
                $removeTagForParent = array() ;
                $productDataAttr = array() ;                
               
                // Handling Special Cases for Product Sheet Creation
                //
                if ( $params->create )
                {
                    if ( isset($p_parameters['Parentage']) )
                            $variant = true ;
                    else    $variant = false ;
                    
                    if ($params->debug)
                        printf('Creation mode: using main category: %s product has variant: %s' . $cr, $p_universe, $variant ? 'Yes' : 'No');
              
                    switch ($p_universe)
                    {
                        case 'Jewelry' :
                            $productData = array();
                            $productData['ClothingType'] = $p_product_type;
                            $productData['Definition'] = $p_universe;
                            break ;
                            
                        case 'ProductClothing' :
                        case 'ClothingAccessories' :
                        case 'Shoes' :
                                
                            $productData = array();
                            $productData['ClothingType'] = $p_product_type;
                            $productData['Definition'] = $p_universe;

                            if ( $variant )
                            {
                              if (isset($combination['Color']) && isset($combination['Size']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  $productData['VariationTheme'] = 'SizeColor';

                                  $productData['Color'] = $combination['Color'];
                                  $productData['ColorMap'] = $combination['ColorMap'];

                                  $productData['Size'] = $combination['Size'];
                                  $productData['SizeMap'] = $combination['SizeMap'];
                              }
                              elseif (isset($combination['Color']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  $productData['VariationTheme'] = 'Color';

                                  $productData['Color'] = $combination['Color'];
                                  $productData['ColorMap'] = $combination['ColorMap'];
                              }
                              elseif (isset($combination['Size']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  $productData['VariationTheme'] = 'Size';

                                  $productData['Size'] = $combination['Size'];
                                  $productData['SizeMap'] = $combination['SizeMap'];
                              }
                            }
                            $removeTagForParent[] = 'Size' ;
                            $removeTagForParent[] = 'Color' ;
                            $removeTagForParent[] = 'ColorMap' ;
                            $removeTagForParent[] = 'SizeMap' ;
                            
                            if ( (isset($combination['Size']) && ! $productData['Size']) && (isset($combination['Color']) && ! $productData['Color']) )
                            {
                                printf($this->l('Unable to export Combination/Variant %d/%d - Field Mapping is maybe missing - Skipping product') . $cr, $details->id, $id_product_attribute);
                                $productData['Variation'] = false ;
                            }                              

                            if ($params->debug)
                                echo 'productData: ' . nl2br(print_r($productData, true)) ;
                            break;
                        default:
                            $productData = array();
                            $productData['ProductType'] = $p_product_type;
                            $productData['Definition'] = $p_universe;
                            
                            if ( $variant )
                            {
                              if (isset($combination['Color']) && isset($combination['Size']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  
                                  if ( $p_universe == 'Sports' )
                                      $productData['VariationTheme'] = 'ColorSize';
                                  elseif ( in_array($p_universe, array('HomeImprovment')) )
                                      $productData['VariationTheme'] = 'Color-Size';
                                  elseif ( in_array($p_universe, array('AutoAccessory', 'Baby', 'Beauty', 'CE', 'Health', 'Home', 'HomeImprovment', 'SWVG', 'ToysBaby', 'WineAndAlcohol')) )
                                      $productData['VariationTheme'] = 'Size-Color';
                                  else
                                      $productData['VariationTheme'] = 'SizeColor';
                                  
                                  $productData['ColorMap'] = $combination['ColorMap'];
                                  $productData['SizeMap'] = $combination['SizeMap'];    
                                  
                                  // Baby > Baby Products Exception
                                  if ( in_array($p_universe, array('Baby')) && in_array($p_product_type, array('BabyProducts')) )
                                  {
                                    $colorIndex = 'ColorName' ;
                                    $sizeIndex  = 'SizeName' ;
                                  }
                                  else
                                  {
                                    $colorIndex = 'Color' ;                                          
                                    $sizeIndex  = 'Size' ;
                                  }

                                  $productData[$colorIndex] = $combination['Color'];
                                  $productData[$sizeIndex]  = $combination['Size'];
                              }
                              elseif (isset($combination['Color']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  $productData['VariationTheme'] = 'Color';

                                  $productData['ColorMap'] = $combination['ColorMap'];
                                  
                                  // Baby > Baby Products Exception
                                  if ( in_array($p_universe, array('Baby')) && in_array($p_product_type, array('BabyProducts')) )
                                    $colorIndex = 'ColorName' ;
                                  else
                                    $colorIndex = 'Color' ;
                                  
                                  $productData[$colorIndex] = $combination['Color'];
                              }
                              elseif (isset($combination['Size']))
                              {
                                  $productData['Variation'] = true;

                                  $productData['Parentage'] = 'child';
                                  $productData['VariationTheme'] = 'Size';

                                  $productData['SizeMap'] = $combination['SizeMap'];
                                  
                                  // Baby > Baby Products Exception
                                  if ( in_array($p_universe, array('Baby')) && in_array($p_product_type, array('BabyProducts')) )
                                    $sizeIndex = 'SizeName' ;
                                  else
                                    $sizeIndex = 'Size' ;
                                  
                                  $productData[$sizeIndex] = $combination['Size'];
                              }
                              
                              if ( (isset($combination['Size']) && ! $productData[$sizeIndex]) && (isset($combination['Color']) && ! $productData[$colorIndex]) )
                              {
                                  printf($this->l('Unable to export Combination/Variant %d/%d - Field Mapping is maybe missing - Skipping product') . $cr, $details->id, $id_product_attribute);
                                  $productData['Variation'] = false ;
                              }                              
                            }
                            $removeTagForParent[] = 'ColorMap' ;
                            $removeTagForParent[] = 'SizeMap' ;
                            
                            if ( isset($sizeIndex) )
                                $removeTagForParent[] = $sizeIndex ;
                            if ( isset($colorIndex) )
                                $removeTagForParent[] = $colorIndex ;
                            
                            if ($params->debug)
                                echo 'productData: ' . nl2br(print_r($productData, true)) ;
                            break;
                    }
                    
                
                    // Incomplete Item
                    //
                    if ( $variant && isset($productData['Variation']) && $productData['Variation'] === false )
                    {
                        $productIndex-- ;
                        continue ;
                    }
                    
                    // Extra Fields - cf xsd_operations.php
                    //
                    if ( $p_extra )
                    {
                        foreach($p_extra['field'] as $name => $value)
                        {
                            // 2013-03-16
                            // Skipping null values
                            if ( ! $value )
                                continue ;
                            
                            // This is a specific variant (eg: Jewelry) description
                            //
                            if ( isset($p_extra['variant']) && isset($p_extra['field']['variant']) )
                            {
                                $profile_variant_name = $p_extra['field']['variant'] ;
                                $profile_variants = $p_extra['variant'][$profile_variant_name] ;

                                if ( $variant )
                                {
                                    $productData['Variation'] = true;
                                    $productData['Parentage'] = 'child';
                                    $productData['VariationTheme'] = $profile_variant_name ;
                                }                                
                                
                                // Associating Variant(Amazon) and Combination(Prestashop)
                                //
                                foreach($profile_variants as $tagName => $psAttribute)
                                {
                                    $mappedValue = null ;
                                    $check = null ;
                                    
                                    // Search matching ps attribute
                                    //
                                    foreach($combinations as $tmpCombination)
                                        if ( $tmpCombination['id_attribute_group'] == $psAttribute )
                                        {
                                            $check = true ; 
                                            break ;
                                        }
                                        
                                    if ( ! $check )   continue ;
                                    
                                    // Search mapped attribute
                                    //
                                    if ( isset($params->mapping_right[$params->id_lang][ $tmpCombination['id_attribute_group'] ] [ $tmpCombination['id_attribute'] ]) )
                                        $mappedValue = $params->mapping_right[$params->id_lang][ $tmpCombination['id_attribute_group'] ] [ $tmpCombination['id_attribute'] ] ;
                                    
                                    if ( ! $mappedValue )   continue ;
                                    
                                    // Found a mapped value for this tag
                                    //
                                    $productData[$tagName] = $mappedValue ;
                                    
                                    $removeTagForParent[] = $tagName ;
                                    
                                    // Found an Amazon tag's attribute for this value
                                    //
                                    if ( isset($p_extra['attributes'][$tagName]) && isset($p_extra['attributes'][$profile_variant_name][$tagName]) && $p_extra['attributes'][$profile_variant_name][$tagName] )
                                        $productDataAttr[$tagName] = $p_extra['attributes'][$profile_variant_name][$tagName] ; 
                                    
                                }
                            }
                            // This is a feature/characteristic
                            //
                            elseif ( isset($p_extra['characteristic'][$name]) && isset($p_extra['field'][$name]) && $p_extra['field'][$name] )
                            {
                                // Already sets - eg: Color
                                if ( isset($productData[$name]) )   break ;
                                
                                $id_feature = $p_extra['field'][$name] ;
                                $value = null ;
                                
                                // Lookup For Features
                                //
                                $features = $details->getFeatures() ;
                                
                                if ( intval($id_feature) && $features )
                                    foreach($features as $feature)
                                        if ( $id_feature == $feature['id_feature'] )
                                                foreach(FeatureValue::getFeatureValueLang($feature['id_feature_value']) as $feature_value)
                                                    if ( $feature['id_feature_value'] == $feature_value['id_feature_value'] && $feature_value['id_lang'] == $id_lang )
                                                        $value = $feature_value['value'] ;
                                
                                // Matched Feature
                                //
                                if ( $features && ($id_feature_value = $feature['id_feature_value']) && $value )
                                {
                                    // Mapped Value
                                    if ( isset($params->features_mapping_right[$id_lang][$id_feature][$id_feature_value]) && $params->features_mapping_right[$id_lang][$id_feature][$id_feature_value] )
                                        $productData[$name] = $params->features_mapping_right[$id_lang][$id_feature][$id_feature_value] ;
                                    // Unmapped Value
                                    else
                                        $productData[$name] = $value ;
                                }    
                            }    
                            // This is a standard extra field with attributes
                            //
                            elseif ( isset($p_extra['attributes'][$name]) )
                            {
                                $productData[$name] = $value ;
                                $productDataAttr[$name] = $p_extra['attributes'][$name] ; 
                            }
                            else
                            {
                                $productData[$name] = $value ;
                            }
                        }
                    }

                    // 2013-03-23 - Added : EAN/UPC Exemption support for products outside Jewelry and Handycraft
                    //
                    if ( $p_code_exemption )
                    {
                        $productData['RegisteredParameter'] = 'PrivateLabel' ;
                    }

                    // 2013-03-29 - Added : AutoAccessory / ItemPackageQuantity - this is required i don't know why ?
                    //
                    if ( $p_universe == 'AutoAccessory' )
                    {
                        $productData['ItemPackageQuantity'] = 1 ;
                    }
                    
                    $productData['Parameters'] = $p_parameters;
                    $productData['Attributes'] = $productDataAttr;

                    // The goal is the first combination is the parent and the subsequent the childrens
                    //
                    if ( $variant )
                    {
                      if ( empty($details->reference) )
                      {
                            if ( ! $params->cronMode ) echo '<span style="color:brown;font-weight:bold;">' ;
                            printf($this->l('Inconsistency for product %d/%d - Is a Combination/Variant and has no Master Reference - Skipping product') . $cr, $details->id, $id_product_attribute);
                            if ( ! $params->cronMode ) echo '</span>' ;                          
                            $skipped++ ;
                            continue ;
                      }
                      if (isset($productData['Variation']) && $productData['Variation'] && !isset($Relationship[$details->reference]))
                      {
                          // Adding children to the relationships - first child
                          $Relationship[$details->reference] = array() ;
                          $Relationship[$details->reference]['parent'] = $details->reference;
                          $Relationship[$details->reference]['children'] = array();
                          $Relationship[$details->reference]['children'][] = $combination['reference'];
                      }
                      elseif ( isset($productData['Variation']) && $productData['Variation'])
                      {
                          // Adding children to the relationships - children
                          $Relationship[$details->reference]['children'][] = $combination['reference'];
                      }
                    }
                }


                // Send Images
                //
                if ($params->images)
                {
                    if ($_SERVER['HTTP_HOST'] == 'localhost' || $_SERVER['HTTP_HOST'] == '127.0.0.1')
                    {
                        printf('Skipping : %d - %s' . "$cr", $details->id, $this->l('Unable to send images from localhost'));
                        $params->images = NULL;
                        break ;
                    }        
                    
                    $productData['ProductImage'] = array();
                    $imageIndex = 1 ;
                    
                    foreach (Amazon_Tools::getProductImages($id_product, $id_product_attribute, $id_lang) as $image)
                    {        
                        $productData['ProductImage'][] = $params->images_url . $image;
                        if ( $imageIndex++ > 8 )
                            break ;
                    }

                    if ($productData['ProductImage'])
                        if ($this->_debug)
                            printf('Products Images: %s' . $cr, print_r($productData['ProductImage'], true));
                }
                
                // Per Category / Formula
                //
                if (!empty($p_formula))
                    $selected_formula = $p_formula;
                elseif ( !empty($params->formula) )
                    $selected_formula = $params->formula;
                else
                    $selected_formula = null ;

                // Price Formula (see Admin > Module > Amazon)
                //
                $stdPrice = $details->getPrice($params->useTax, ($id_product_attribute ? $id_product_attribute : null), 6, NULL, false, ! $details->on_sale && $params->specials) ;
                $newPrice = Amazon_Tools::Formula($stdPrice, $selected_formula);

                $sales = array() ;
                
                // Apply Sales for PS > 1.4
                //
                if ( version_compare(_PS_VERSION_, '1.4', '>=') && $params->specials )
                {
                    $specificPrice = SpecificPrice::getSpecificPrice($id_product, $params->fromCurrency->id, $params->id_shop, (int)Amazon::configurationGet('PS_COUNTRY_DEFAULT'), (int)_PS_DEFAULT_CUSTOMER_GROUP_, 1, $id_product_attribute, 0, 0, 1) ;

                    // Sales
                    //
                    if ( $details->on_sale && $specificPrice && isset($specificPrice['reduction_type']) && isset($specificPrice['from']) && isset($specificPrice['to']) && intval($specificPrice['from']) && intval($specificPrice['to']) )
                    {
                        // ISO 8601
                        $dateStart    = date('c', strtotime($specificPrice['from'])) ;
                        $dateEnd      = date('c', strtotime($specificPrice['to'])) ;
                        
                        $salePrice    = $details->getPrice($params->useTax, ($id_product_attribute ? $id_product_attribute : null), 6, NULL, false, $params->specials) ; 

                        if ($params->fromCurrency->iso_code != $params->toCurrency->iso_code)                
                            $salePrice = Tools::convertPrice($salePrice, $params->toCurrency);          

                        $salePrice = Amazon_Tools::CallBack($salePrice, $params->callback) ;

                        printf('%s : %s - %s : %s %s %s %s' . $cr, $this->l('Apply Sale Price'), $dateStart, $dateEnd, $this->l('Old Price'), Tools::displayPrice($stdPrice),  $this->l('Sale Price'), $salePrice);                          

                        $sales['dateStart']   = $dateStart ;
                        $sales['dateEnd']     = $dateEnd ;
                        $sales['salePrice']   = $salePrice ;
                    }
                }
                if ($params->debug)
                    printf('Apply formula - price : %s was : %s' . $cr, Tools::displayPrice($newPrice), Tools::displayPrice($stdPrice));

                // Override Price with Price/Strategy (Option)
                //

                if (defined('AMAZON_TRADER') && AMAZON_TRADER)
                {
                    $strategy = new Amazon_Strategy($combination['reference'], true, $params->id_lang);

                    if ($strategy->lookup())
                        if (($stratege = $strategy->get()))
                            $newPrice = Amazon_Tools::Formula($stratege['actual_price'], $selected_formula);
                }
 
                // Product Options
                //
                $options = Amazon_Product::getProductOptions($id_product, $params->id_lang);

                $disabled = $options['disable'] ? true : false;
                $force = $options['force'] ? true : false;
                $text = $options['text'];
                $latency = intval($options['latency']);
                $shipping = $options['shipping'];
                $shipping_type = intval($options['shipping_type']);
                $nopexport = $options['nopexport'] ? $options['nopexport'] : $params->stockOnly;
                $noqexport = $options['noqexport'];
                $fba = $options['fba'] ? true : false ;
             
                // Synchronizing by ASIN (ASIN override)
                //
                if ($params->asinHasPriority && isset($options['asin1']) && !empty($options['asin1']))
                    $params->asinOverride = true;
                else
                    $params->asinOverride = false;

                if ($params->debug && $params->asinOverride)
                    printf('Using ASIN override mode' . $cr);

                // Using Fulfilment By Amazon
                //
                if ($params->debug && $fba)
                    printf('Using FBA' . $cr);                
                
                if (!empty($options['price']) && floatval($options['price']))
                    $newPrice = floatval($options['price']);

                // Convert to platform currency
                //
                if ($params->fromCurrency->iso_code != $params->toCurrency->iso_code)
                {
                    $oldPrice = $newPrice;
                    $newPrice = Tools::convertPrice($newPrice, $params->toCurrency);

                    if ($params->debug)
                        printf('Convert from currency: %s to currency: %s - price is: %s and was %s' . $cr, $params->fromCurrency->iso_code, $params->toCurrency->iso_code, $newPrice, $oldPrice);
                }

                // Price CallBack (see Admin > Module > Amazon)
                //
                $newPrice = Amazon_Tools::CallBack($newPrice, $params->callback);

                // Out of Stock policies
                //
                if ($quantity < $params->outOfStock && isset($details->out_of_stock) && !Product::isAvailableWhenOutOfStock($details->out_of_stock) && !$force)
                {
                    $quantity = 0;
                }
                elseif ($quantity < $params->outOfStock && !$force)
                {
                    $quantity = 0;
                }
                elseif ($force)
                {
                    $quantity = 999;
                }

                if (!$details->active || $disabled)
                {
                    $quantity = 0;
                }
                

                if ( ! $params->create && $params->deleteProducts && (($quantity == 0 && ($combination[$ps_code] || $params->asinOverride) ) || $disabled) || $action == Amazon::Remove)
                {
                    $productsDelete[$d]['SKU'] = $combination['reference'];

                    if (!isset($productsDelete[$d]['SKU']) || empty($productsDelete[$d]['SKU']))
                    {
                        if ($params->debug)
                            printf('Skipping : %d - Has No SKU !' . "$cr", $details->id_product);
                        $skipped++;
                        continue;
                    }

                    if ($params->asinOverride)
                    {
                        $productsDelete[$d]['ProductIDType'] = 'ASIN';
                        $productsDelete[$d]['ProductIDCode'] = trim($options['asin1']);
                    }
                    elseif ($ps_code != 'reference')
                    {
                        $productsDelete[$d]['ProductIDType'] = $az_code;
                        $productsDelete[$d]['ProductIDCode'] = trim($combination[$ps_code]);
                    }
                    if ($action == Amazon::Remove)
                        $reason = 'Deleted';
                    else
                        $reason = 'OutOfStock';

                    printf('Delete() : %s %s %s - %s' . "$cr", isset($productsDelete[$d]['ProductIDType']) ? $productsDelete[$d]['ProductIDType'] : '', $productsDelete[$d]['ProductIDCode'], $productsDelete[$d]['SKU'], $reason);

                    $productIndex-- ;
                    $d++;
                }
                elseif ($action == Amazon::Update || $action == Amazon::Add)
                {
                    $productsUpdate[$u]['SKU'] = trim($combination['reference']);

                    if (!isset($productsUpdate[$u]['SKU']) || empty($productsUpdate[$u]['SKU']))
                    {
                        if ($params->debug)
                            printf('Skipping : %d - Has No SKU !' . "$cr", $details->id_product);
                        $skipped++;
                        continue;
                    }

                    // If this is a new product, quantity = 0 and deleteProducts is sets to On (configuration)
                    // Do not export the product to delete it the next time !
                    //
                    if ($action == Amazon::Add && $params->deleteProducts && !$quantity && ! $params->create)
                    {
                        if ($params->debug)
                            printf('Skipping : %d - Has a quantity of 0 and deleteProducts is sets to on' . "$cr", $details->id_product);
                        $skipped++;
                        continue;
                    }

                    if ($params->asinOverride)
                    {
                        $productsUpdate[$u]['ProductIDType'] = 'ASIN';
                        $productsUpdate[$u]['ProductIDCode'] = trim($options['asin1']);
                    }
                    elseif ($ps_code != 'reference')
                    {
                        $productsUpdate[$u]['ProductIDType'] = $az_code;
                        $productsUpdate[$u]['ProductIDCode'] = trim($combination[$ps_code]);
                    }
                    $productCondition = isset($details->condition) && !empty($details->condition) && isset($params->conditionMap[$details->condition]) && !empty($params->conditionMap[$details->condition]) ? $params->conditionMap[$details->condition] : 'New';

                    $productsUpdate[$u]['ConditionType'] = $productCondition;
                    $productsUpdate[$u]['ConditionNote'] = $text;
                    $productsUpdate[$u]['Quantity'] = $quantity;
                    $productsUpdate[$u]['Price'] = $newPrice;
                    $productsUpdate[$u]['NoPriceExport'] = $nopexport ? true : false;
                    $productsUpdate[$u]['NoQtyExport'] = (! $params->create && $noqexport) ? true : false;
                    $productsUpdate[$u]['Sales'] = $sales ;
                    
                    // Product Sheet Elements
                    //
                    if ($params->sendProductDescription)
                    {
                        if (!isset($productsUpdate[$u]['ProductDescription']))
                            $productsUpdate[$u]['ProductDescription'] = array();

                        $master_name = trim(substr($details->name, 0, 500 - 1));

                        if ( isset($details->name_attributes) && ! empty($details->name_attributes) && $id_product_attribute )
                                $standard_name = trim(substr(sprintf('%s - %s', $details->name, rtrim($details->name_attributes, ', ')), 0, 500 - 1));
                        else    $standard_name = $master_name ;
                        
                        switch($params->titleFormat)
                        {
                            case Amazon::FormatManufacturerTitle :
                                if ( ! $manufacturer )
                                {
                                    $name = $standard_name ;
                                    break ;
                                }
                                $master_name = trim(substr(sprintf('%s - %s', $manufacturer, $details->name), 0, 500 - 1));
                                
                                if ( isset($details->name_attributes) && ! empty($details->name_attributes) && $id_product_attribute )
                                        $name = trim(substr(sprintf('%s - %s - %s', $manufacturer, $details->name, rtrim($details->name_attributes, ', ')), 0, 500 - 1));
                                else    $name = $master_name ;                                
                                break ;
                                
                            case Amazon::FormatManufacturerTitleReference :
                                if ( ! $manufacturer )
                                {
                                    $name = $standard_name ;
                                    break ;
                                }
                                $master_name = trim(substr(sprintf('%s - %s - %s', $manufacturer, $details->name, trim($details->reference)), 0, 500 - 1));  
                                
                                if ( isset($details->name_attributes) && ! empty($details->name_attributes) && $id_product_attribute )
                                        $name = trim(substr(sprintf('%s - %s - %s - %s', $manufacturer, $details->name, trim($details->reference), rtrim($details->name_attributes, ', ')), 0, 500 - 1));
                                else    $name = $master_name ;                              
                                break ;                                
                            default :
                                $name = $standard_name ;
                                break ;                                
                        }
                        
                        $productsUpdate[$u]['ProductDescription']['Title'] = $name ;
                        $productsUpdate[$u]['ProductDescription']['Description'] = substr(Amazon_Tools::strip_html(trim($details->{$params->descriptionField})), 0, 2000 - 1);
                    }

                    // Send Images
                    //
                    if ( $params->images && $productData['ProductImage'] )
                    {
                        if (!isset($productsUpdate[$u]['ProductData']))
                            $productsUpdate[$u]['ProductData'] = array();

                        $productsUpdate[$u]['ProductData'] = $productData;
                    }
                    // Product Sheet Creation
                    //
                    if ( $profile_name && $params->create )
                    {
                        if ($productData)
                        {
                            if (!isset($productsUpdate[$u]['ProductData']))
                                $productsUpdate[$u]['ProductData'] = array();

                            $productsUpdate[$u]['ProductData'] = $productData;
                        }

                        if (!isset($productsUpdate[$u]['ProductDescription']))
                            $productsUpdate[$u]['ProductDescription'] = array();

                        if ($manufacturer)
                            $productsUpdate[$u]['ProductDescription']['Manufacturer'] = $manufacturer;
                        else
                            printf( $this->l('Skipping : %d - Item creation might be skipped - Brand is not set') . "$cr", $details->id);
                            
                        // In most cases it is same than manufacturer
                        if ($manufacturer)
                            $productsUpdate[$u]['ProductDescription']['Brand'] = $manufacturer;

                        // 2013/03/16
                        // Arnaud Lempereur / Amazon France said:
                        // MfrPartNumber is not mandatory
                        // 
                        // if ($manufacturer && $details->supplier_reference)
                        //    $productsUpdate[$u]['ProductDescription']['MfrPartNumber'] = $details->supplier_reference ;
                        // 2013/03/22
                        // Arnaud Lempereur / Amazon France said: MfrPartNumber is only mandatory for Office > PaperProducts
                        //
                        if ( isset($p_product_type) && $p_product_type == 'PaperProducts' )
                        {
                            if ( version_compare(_PS_VERSION_,'1.5','>=') )
                                $supplier_reference = ProductSupplier::getProductSupplierReference($id_product, $id_product_attribute, $details->id_supplier) ;
                            else
                                $supplier_reference = $details->supplier_reference ;

                            $productsUpdate[$u]['ProductDescription']['MfrPartNumber'] = $supplier_reference ;
                        }

                        if ($p_browsenode)
                            $productsUpdate[$u]['ProductDescription']['RecommendedBrowseNode'] = $p_browsenode;
                        
                        if ( $params->weightUnit && (float)$details->weight )
                            switch($params->weightUnit)
                            {
                                case 'GR' : 
                                case 'KG' : 
                                case 'OZ' : 
                                case 'LB' : 
                                case 'MG' : 
                                    $unit   = $params->weightUnit ;
                                    $value  = number_format((float)$details->weight, 2) ;
                                    
                                    $productsUpdate[$u]['ProductDescription']['PackageWeight'] = $value ;
                                    $productsUpdate[$u]['ProductDescription']['PackageWeightUnit'] = $unit ;
                                    
                                    // Note: maybe we have to add handling weight/packing weight to this value
                                    $productsUpdate[$u]['ProductDescription']['ShippingWeight'] = $value ;
                                    $productsUpdate[$u]['ProductDescription']['ShippingWeightUnit'] = $unit ;                                    
                                    break ;
                                // case another one : change the value to a known amz value
                                default :
                                    printf( $this->l('Warning : %d - Amazon couldn\'t match this weight unit: %s for product: %d') . "$cr", $params->weightUnit, $details->id);
                                    break ;
                            }
                    }
                        
                    // Shipping Override
                    //
                    if ($shipping !== NULL && is_numeric($shipping))
                    {
                        $productsUpdate[$u]['ShippingPrice'] = $shipping;
                        $productsUpdate[$u]['ShippingType'] = $shipping_type == 2 ? $params->shippingOverrideExps : $params->shippingOverrideStds;
                        $shipping_text = sprintf('- ' . $this->l('Overriding Shipping Fees: %s'), $shipping);
                    }
                    elseif ($shipping === NULL)
                    {
                        $productsUpdate[$u]['ShippingPrice'] = '';
                        $productsUpdate[$u]['ShippingType'] = $shipping_type == 2 ? $params->shippingOverrideExps : $params->shippingOverrideStds;
                        
                        if ($params->debug)
                            $shipping_text = sprintf('- ' . $this->l('Overriding Shipping Fees: void'), $shipping);
                        else
                            $shipping_text = '' ;
                    }
                    else
                        $shipping_text = NULL;
                    
                    // Options
                    //
                    if ($latency)
                        $productsUpdate[$u]['FulfillmentLatency'] = $latency;
                    
                    if ( $fba )
                        $productsUpdate[$u]['FBA'] = $params->fullfillmentCenterId ;
                 
                    $actionType = $profile_name && $params->create ? $this->l('Create') : $this->l('Update') ;

                    printf('%s: %s %s %s - ' .  $this->l('Qty') . ': %s - ' .  $this->l('Price') . ': %s %s ' .  $this->l('was') . ' %s %s' . "$cr", $actionType, isset($productsUpdate[$u]['ProductIDType']) ? $productsUpdate[$u]['ProductIDType'] : null, isset($productsUpdate[$u]['ProductIDCode']) ? $productsUpdate[$u]['ProductIDCode'] : null, $productsUpdate[$u]['SKU'], isset($productsUpdate[$u]['Quantity']) ? $productsUpdate[$u]['Quantity'] : 'n/a', $params->toCurrency->iso_code, $newPrice, Tools::displayPrice($stdPrice), $shipping_text);

                    // Create the master product for Variations
                    //
                    if ( $params->create && $masterProduct && $variant && $productIndex == 1 )
                    {
                        $firstProduct = $productsUpdate[$u];

                        $productsUpdate[$u]['SKU'] = $details->reference;
                        $productsUpdate[$u]['ProductData']['Parentage'] = 'parent';
                        $productsUpdate[$u]['ProductDescription']['Title'] = $master_name ;

                        // Unset unwanted tags for parent
                        //
                        foreach($removeTagForParent as $tagName)
                            unset($productsUpdate[$u]['ProductData'][$tagName]) ;
                        
                        foreach(array('ConditionType', 'ConditionNote', 'Quantity', 'Price', 'ShippingPrice', 'ShippingType', 'ProductIDType', 'ProductIDCode') as $tagName)
                            unset($productsUpdate[$u][$tagName]) ;
                        
                        // Do not export quantity and price for parent
                        $productsUpdate[$u]['NoPriceExport'] = true;
                        $productsUpdate[$u]['NoQtyExport'] = true;

                        $u++;

                        $productsUpdate[$u] = $firstProduct;
                    }
                    $u++;
                    
                }
            }
            $i++;
        }
        if ( ! $params->cronMode )  echo "<br /><br />" ;
        else echo $cr . $cr ;

        // For customer debug purpose
        // 
        if ( $params->xmlOnly )
        {
            $this->_amazonApi->MWS_Action = Service::MWS_DO_NOT_SEND ;
            $this->_amazonApi->displayXML = true ;
        }
        
        if (!count($productsUpdate))
        {
            printf($this->l('No products to send on Amazon') . $cr);
        }
        else
        {
            if ($params->debug && isset($productsUpdate))
            {
                $productDebug = $productsUpdate ;
                unset($productDebug['ProductData']) ;
                
                echo nl2br(print_r($productDebug, true));
            }
            if ($params->sendToAmazon)
            {
                if ( $params->relationShipsOnly )
                {
                    printf($this->l('Updating relationships only - %s items') . $cr, count($productsUpdate));

                    if (!$datas = $this->_amazonApi->updateRelationships($Relationship))
                    {
                        if ( ! $params->xmlOnly )
                        {
                            echo '<span style="width:100%;display:inline-block;background-color:yellow;color:red;">';
                            printf('%s/%s: ' . $this->l('Error : query failed (%s)' . $cr), basename(__FILE__), __LINE__, print_r($datas, true));
                            echo '</span>';
                        }
                    }                    
                }
                else
                {
                    printf($this->l('%s products to synchronize with Amazon') . $cr, count($productsUpdate));

                    // Operation Override
                    if ( $params->delete )
                        $this->_amazonApi->setOperationMode( Service::OPERATIONS_DELETE ) ;
                        
                    if (!$datas = $this->_amazonApi->updateProducts($productsUpdate, $Relationship))
                    {
                        if ( ! $params->xmlOnly )
                        {
                            echo '<span style="width:100%;display:inline-block;background-color:yellow;color:red;">';
                            printf('%s/%s: ' . $this->l('Error : query failed (%s)' . $cr), basename(__FILE__), __LINE__, print_r($datas, true));
                            echo '</span>';
                        }
                    }
                }
                if ($params->debug)
                    echo nl2br(print_r($datas));

                $mwsSuccess = true ;

                if (isset($datas['products']) && intval($datas['products']))
                {
                    $mwsSuccess = $mwsSuccess && true;
                    printf($this->l('Products Feed - Submission ID: %s') . $cr, $datas['products']);
                }
                else
                    $mwsSuccess = $mwsSuccess && false;

                if (isset($datas['inventory']) && intval($datas['inventory']))
                {
                    $mwsSuccess =  $mwsSuccess && true ;
                    printf($this->l('Inventory Feed - Submission ID: %s') . $cr, $datas['inventory']);
                }
                else
                    $mwsSuccess = $mwsSuccess && false;

                if (isset($datas['prices']) && intval($datas['prices']))
                {
                    $mwsSuccess =  $mwsSuccess && true ;
                    printf($this->l('Price Feed - Submission ID: %s') . $cr, $datas['prices']);
                }
                else
                    $mwsSuccess = $mwsSuccess && false;

                if (isset($datas['overrides']) && intval($datas['overrides']))
                {
                    $mwsSuccess =  $mwsSuccess && true ;
                    printf($this->l('Overrides Feed - Submission ID: %s') . $cr, $datas['overrides']);
                }

                if (isset($datas['images']) && intval($datas['images']))
                {
                    $mwsSuccess =  $mwsSuccess && true ;
                    printf($this->l('Images Feed - Submission ID: %s') . $cr, $datas['images']);
                }

                if (isset($datas['relations']) && intval($datas['relations']))
                {
                    $mwsSuccess =  $mwsSuccess && true ;
                    printf($this->l('Relationship Feed - Submission ID: %s') . $cr, $datas['relations']);
                }

                // We sucessfully exports to Amazon
                //
                if ($mwsSuccess === true)
                {
                    printf($this->l('Products was successfully submitted to Amazon...') . $cr);

                    // Update Marketplace Product Action Table
                    //
                    if ($params->updateDateStart && $params->updateDateStop)
                    {
                        Amazon_Product::marketplaceActionAcknowledgde($params->id_lang, $params->updateDateStart, $params->updateDateStop, $dateAcknowledgde = $params->currentDate);
                    }

                    // Save for report
                    //
                    $submissions = unserialize(base64_decode(Amazon::configurationGet('AMAZON_LAST_SUBMISSIONS')));
                    $submissions[$params->id_lang] = $datas;

                    Amazon::configurationUpdateValue('AMAZON_LAST_SUBMISSIONS', base64_encode(serialize($submissions)));
                }
                else
                    printf($this->l('Products: Nothing to Update !') . $cr);
            }
            else
            {
                printf($this->l('%s products to synchronize with Amazon') . $cr, count($productsUpdate));
            }
        }

        if (!count($productsDelete))
        {
            printf($this->l('No products to delete') . $cr);
        }
        elseif ($params->deleteProducts)
        {
            if ($params->debug)
                echo nl2br(print_r($productsDelete, true));

            if ($params->sendToAmazon)
            {
                if (!($datas = $this->_amazonApi->deleteProducts($productsDelete)))
                {
                    if ( ! $params->xmlOnly )
                    {
                        printf($this->l('%s/%s: ' . 'Error : query failed (%s)' . $cr), basename(__FILE__), __LINE__, print_r($datas));
                    }
                    // Oooop's something happen - revert the promise
                    //
                    if ($params->updateDateStart && $params->updateDateStop)
                        Amazon_Product::marketplaceActionAcknowledgde($params->id_lang, $params->updateDateStart, $params->updateDateStop, $dateAcknowledgde = $params->currentDate, true);
                }

                printf($this->l('%s deleted products') . $cr, count($productsDelete));
            }
            else
            {
                printf($this->l('%s products to delete on Amazon') . $cr, count($productsDelete));
            }
        }

        if ($skipped)
        {
            printf($this->l('%d skipped products' . $cr), $skipped);
        }
        

        if (!$i)
        {
            printf($this->l('No products to export !') . $cr);
        }
        elseif ( $params->sendToAmazon && ! $params->xmlOnly )
        {
            printf($this->l('%s products sent') . $cr, count($productsUpdate));
        }
        if ( isset($duplicate) && $duplicate ) 
            printf($this->l('%s duplicate products') . $cr, $duplicate);
            
        if ( count($productsUpdate) || count($productsDelete) && $params->sendToAmazon )
        {
            if ($params->cronMode)
                Amazon::configurationUpdateValue('AMAZON_LAST_CRON_UPDATED_' . $params->id_lang, $params->currentDate);
            else
                Amazon::configurationUpdateValue('AMAZON_LAST_UPDATED_' . $params->id_lang, $params->currentDate);
        }
    }

}

class Params extends Amazon_ExportProducts
{

    protected $debug;
    protected $outOfStock;
    protected $formula;
    protected $callback;
    protected $synchField;
    protected $specials;
    protected $useTax;
    protected $conditionMap;
    protected $shippingOverrideStds;
    protected $shippingOverrideExps;
    protected $deleteProducts;
    protected $stockOnly;
    protected $asinHasPriority;
    public $categories;
    public $cronMode = false;
    public $sendProductDescription = false;
    public $relationShipsOnly = false;
    public $xmlOnly = false;
    public $sendActives = true;
    public $action;
    public $syncBothMode = false;
    public $sendToAmazon = false;
    public $id_lang;
    public $europe = 0;
    public $platform;
    public $toCurrency;
    public $fromCurrency;
    public $updateDateStart = false;
    public $updateDateStop = false;
    protected $currentDate;
    protected $profile;
    protected $profile2category;
    protected $images;
    protected $create;
    protected $mapping_left;
    protected $mapping_right;
    protected $dateSince;
    protected $weightUnit;
    protected $descriptionField ;
    public $asinOverride = false ;
    
    protected $fullfillmentCenterId ;
    protected $marketplaceID ;
    protected $id_shop ;
    protected $id_warehouse = null ;
    
    // For Converting Currencies
    //
    public function __construct($id_lang)
    {
        $this->id_lang = $id_lang;

    $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG')));
        $this->debug = intval($params) ? true : false;
        // Stock
        //
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_OUT_OF_STOCK')));
        $this->outOfStock = $params[$id_lang];

        // Price Formula & Callback
        //
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_PRICE_FORMULA')));
        $this->formula = $params[$id_lang];

        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_PRICE_CALLBACK')));
        $this->callback = $params[$id_lang];

        $this->CategoriesFormulas = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CATEGORIES_FORMULA')));

        // Field used for synchronisation - EAN / UPC / SKU and why not ISBN ...
        //
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_FIELD')));
        $this->synchField = $params[$id_lang];

        // Use Reduc / specials price
        //
        $this->specials = intval(unserialize(base64_decode(Amazon::configurationGet('AMAZON_SPECIALS')))) ? true : false;
        $this->useTax = intval(unserialize(base64_decode(Amazon::configurationGet('AMAZON_TAXES')))) ? true : false;

        // State / Condition
        //
        $this->conditionMap = array_flip(unserialize(base64_decode(Amazon::configurationGet('AMAZON_CONDITION_MAP'))));

        // Shipping Overrides
        //
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_SHIPPING_OVERRIDES_STD')));
        $this->shippingOverrideStds = isset($params[$id_lang]) ? $params[$id_lang] : '';

        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_SHIPPING_OVERRIDES_EXP')));
        $this->shippingOverrideExps = isset($params[$id_lang]) ? $params[$id_lang] : '';

        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DELETE_PRODUCTS')));
        $this->deleteProducts = intval($params) ? true : false;

        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_STOCK_ONLY')));
        $this->stockOnly = intval($params) ? true : false;

        // Title Formatage
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_TITLE_FORMAT')));
        $this->titleFormat = $params ;
        
        // Brute Force, as as named !
        //
        $this->bruteForce = unserialize(base64_decode(Amazon::configurationGet('AMAZON_BRUTE_FORCE')));

        // Saved Categories
        //
        $this->categories = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CATEGORIES')));

        // Profiles
        //
        $this->profile = unserialize(base64_decode(Amazon::configurationGet('AMAZON_PROFILES')));
        $this->profile2category = unserialize(base64_decode(Amazon::configurationGet('AMAZON_PROFILES_CATEGORIES')));

        // ASIN
        //
        $params = unserialize(base64_decode(Amazon::configurationGet('AMAZON_USE_ASIN')));
        $this->asinHasPriority = isset($params[$id_lang]) && intval($params[$id_lang]) ? true : false;

        $this->currentDate = date('Y-m-d H:i:s');

        $this->images = intval(Tools::getValue('images'));
        $this->create = intval(Tools::getValue('create'));
        $this->delete = intval(Tools::getValue('delete'));

        // Absolute URL to images
        //
        $baseurl = 'http://' . htmlspecialchars($_SERVER['HTTP_HOST'], ENT_COMPAT, 'UTF-8') . __PS_BASE_URI__;
        $this->images_url = $baseurl . 'img/p/';

        // Attributes Mapping (Color, Size..)
        // 
        $this->mapping_left = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ATTRIBUTES_MAPPING_L')));
        $this->mapping_right = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ATTRIBUTES_MAPPING_R')));

        // Attributes Mapping (Color, Size..)
        // 
        $this->features_mapping_left = unserialize(base64_decode(Amazon::configurationGet('AMAZON_FEATURES_MAPPING_L')));
        $this->features_mapping_right = unserialize(base64_decode(Amazon::configurationGet('AMAZON_FEATURES_MAPPING_R')));
        
        $this->dateSince = Tools::getValue('since') ;
        
        $this->weightUnit = strtoupper(preg_replace('/[^A-Za-z]/', '', Amazon::configurationGet('PS_WEIGHT_UNIT'))) ;
        
        $params = Amazon::configurationGet('AMAZON_DESCRIPTION_FIELD') ;
        $this->descriptionField = $params ? $params : Amazon::FieldDescriptionLong ;
        
        // Shop (PS 1.5)
        //
        $this->id_shop = (int)Amazon::configurationGet('AMAZON_SHOP') ;
        $this->id_shop = $this->id_shop ? $this->id_shop : 1 ;
        
        $this->id_warehouse = (int)Amazon::configurationGet('AMAZON_WAREHOUSE') ;
    }

}

$amazonExportProduct = new Amazon_ExportProducts;
$amazonExportProduct->DoIt();

?>