<?php


@require_once(dirname(__FILE__).'/../../../config/config.inc.php');
@require_once(dirname(__FILE__).'/../../../init.php');

require_once(dirname(__FILE__).'/../amazon.php');

require_once(dirname(__FILE__).'/../classes/amazon.webservice.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.product.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.tools.class.php');

class Amazon_ProductReport extends Module
{
  private $_debug = false ;
  private $_europe ;

  private $module   = 'amazon';
  public $name   = 'amazon';

  function __construct()
  {
    global $cookie, $_MODULE, $_LANGADM;

    $this->id_lang = (int)Tools::getValue('id_lang');

    if ( ! $this->id_lang ) 
        $this->id_lang = Configuration::get('PS_LANG_DEFAULT') ;    
    
    $langFile = _PS_MODULE_DIR_.$this->module.'/'.Language::getIsoById($this->id_lang).'.php';
    $mySelf   = basename(__FILE__, '.php') ;

    if(file_exists($langFile))
    {
      require_once $langFile;
      foreach($_MODULE  as $key=>$value)
          $_MODULE[str_replace($mySelf . '_' , $this->name . '_', $key)] = $value;
    }
    $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/' ;
    $this->images = $module . 'images/';

    $this->_amazon = new Amazon() ;

    parent::__construct() ;
  }
  public function l($string, $specific = false, $id_lang = null)
  {
      return( $this->_amazon->l($string, $specific = false, $id_lang = null) ) ;
  }


  function DoIt()
  {
    global $cookie, $cart ;
    ob_start() ;

    $id_lang = intval(Tools::getValue('amazon_lang')) ;

    $callback   = Tools::getValue('callback') ;
    $action     = strval(Tools::getValue('action')) ;
    $cr = "<br />\n";

    $this->_debug = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG'))) ;

    // Init
    //
    $amazon = Amazon_Tools::selectPlatforms($id_lang, $this->_debug) ;


    if ( $this->_debug )
        echo nl2br(print_r($amazon['auth'], true) .print_r($amazon['params'], true) . print_r($amazon['platforms'], true));

    if ( ! ($this->_amazonApi = new Service($amazon['auth'], $amazon['params'], $amazon['platforms'], $this->_debug)) )
    {
      echo $this->l('Unable to login') . $cr ;
      die;
    }

    switch($action)
    {
        case 'submission-report' :
            $html = '' ;
            $pass = false ;
            $submissions = unserialize(base64_decode(Amazon::configurationGet('AMAZON_LAST_SUBMISSIONS'))) ;


            // Product Submission Report
            //
            if ( isset($submissions[$id_lang]['products']) && intval($submissions[$id_lang]['products']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['products']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Product Feed'), $submissions[$id_lang]['products'])) ;
                  $pass = true;
              }
            }

            // Inventory Submission Report
            //
            if ( isset($submissions[$id_lang]['inventory']) && intval($submissions[$id_lang]['inventory']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['inventory']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Inventory Feed'), $submissions[$id_lang]['inventory'])) ;
                  $pass = true;
              }
            }

            // Prices Submission Report
            //
            if (  isset($submissions[$id_lang]['prices']) && intval($submissions[$id_lang]['prices']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['prices']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Prices Feed'), $submissions[$id_lang]['prices'])) ;
                  $pass = true;
              }
            }

            // Override Submission Report
            //
            if (  isset($submissions[$id_lang]['overrides']) && intval($submissions[$id_lang]['overrides']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['overrides']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Overrides Feed'), $submissions[$id_lang]['overrides'])) ;
                  $pass = true;
              }
            }

            // Images Submission Report
            //
            if (  isset($submissions[$id_lang]['images']) && intval($submissions[$id_lang]['images']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['images']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Images Feed'), $submissions[$id_lang]['images'])) ;
                  $pass = true;
              }
            }
            
            // Relationship Submission Report
            //
            if (  isset($submissions[$id_lang]['relations']) && intval($submissions[$id_lang]['relations']) )
            {
              $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['relations']) ;

              if ( $response !== false && $response !== null )
              {
                  $html .= $this->displayGetFeedSubmissionResult($response, sprintf('%s (%s)', $this->l('Relationship Feed'), $submissions[$id_lang]['relations'])) ;
                  $pass = true;
              }
            }
            
            // Debug Messages
            //
            if ( ! $pass )
                echo $this->l('No available reports') . $cr ;

            $output = '<div class="report-debug-div">' ;
            $output .= ob_get_clean() ;
            $output .= '</div>' ;

            echo $html ;

            echo $output ;

            break ;

    }
  }
  private function displayGetFeedSubmissionResult($obj, $title)
  {
    $html = '' ;
    $errors = '';


    if ( isset($obj->Message->ProcessingReport->Result->ResultCode) && ($obj->Message->ProcessingReport->Result->ResultCode == 'Warning' || $obj->Message->ProcessingReport->Result->ResultCode == 'Error') )
    {
        if (is_object($obj->Message->ProcessingReport->Result))
        foreach($obj->Message->ProcessingReport->Result as $result)
        {
          if ( isset($result->AdditionalInfo->SKU) )
              $additionnal_information = sprintf('SKU: %s - ', $result->AdditionalInfo->SKU) ;
          else
              $additionnal_information = '' ;

            $errors .= nl2br(sprintf('Error %d: %s%s' . "\n\n",
                $result->ResultMessageCode,
                $additionnal_information,    
                $result->ResultDescription)) ;
        }
      else
      {
          if ( isset($result->AdditionalInfo->SKU) )
              $additionnal_information = sprintf('SKU: %s - ', $result->AdditionalInfo->SKU) ;
          else
              $additionnal_information = '' ;

            $errors .= nl2br(sprintf('Error %d: %s%s' . "\n\n",
                $obj->Message->ProcessingReport->Result->ResultMessageCode,
                $additionnal_information,        
                $obj->Message->ProcessingReport->Result->ResultDescription)) ;
      }

    }
    elseif ( isset($obj->Message->ProcessingReport->StatusCode) && $obj->Message->ProcessingReport->StatusCode == 'Complete' )
    {
       $errors = '' ;
    }
    else
    {
       $errors .= nl2br(print_r($obj, true)) ;    
    }

    $html .= '
              <span class="submission-result-title">'. $title . '</span><br />' ;

    if ( isset($obj->Message) && isset($obj->Message->ProcessingReport->ProcessingSummary) )
    {
      $summary = $obj->Message->ProcessingReport->ProcessingSummary ;

      $html .= '
              <table class="submission-results-table">
                  <thead>
                      <tr>
                          <th>' . $this->l('Entry') . '</th>
                          <th>' . $this->l('Result') . '</th>
                      </tr>
                  </thead>
                  <tbody>
                     <tr>
                          <td  class="col1">' . $this->l('Entries Processed') . '</td>
                          <td style="font-weight:bold;">' . strval($summary->MessagesProcessed) . '</td>
                     <tr>
                     </tr>
                          <td  class="col1">' . $this->l('Entries Successful') . '</td>
                          <td style="color:#1D910F;font-weight:bold;">' . strval($summary->MessagesSuccessful) . '</td>
                     <tr>
                     </tr>
                          <td class="col1">' . $this->l('Entries with Error') . '</td>
                          <td style="color:#FD0000;font-weight:bold;">' . strval($summary->MessagesWithError) . '</td>
                     <tr>
                     </tr>
                          <td class="col1">' . $this->l('Entries with Warning') . '</td>
                          <td style="color:#FFA300;font-weight:bold;">' . strval($summary->MessagesWithWarning) . '</td>
                     </tr>
                     </tr>
                          <td class="col1">' . $this->l('Messages Logs') . '</td>
                          <td>' . $errors . '</td>
                     </tr>
                  </tbody>
              </table>
              ' ;
    }
    /*
    SimpleXMLElement Object
(
[Error] => SimpleXMLElement Object
(
[Type] => Sender
[Code] => FeedProcessingResultNotReady
[Message] => Feed Submission Result is not ready for Feed Submission Id: 3899441890
[Detail] => SimpleXMLElement Object
(
)
*/
    elseif ( isset($obj->Error) )
    {
      $error = $obj->Error->Message ;

      $html .= '
              <table class="submission-results-table">
                  <thead>
                      <tr>
                          <th>' . $this->l('Entry') . '</th>
                          <th>' . $this->l('Result') . '</th>
                      </tr>
                  </thead>
                     </tr>
                          <td class="col1">' . $this->l('Error') . '</td>
                          <td style="color:#FFA300;font-weight:bold;">' . strval($obj->Error->Code) . '</td>
                     </tr>
                  </body>
                     </tr>
                          <td class="col1">' . $this->l('Messages Logs') . '</td>
                          <td>' . $error . '</td>
                     </tr>
                  </tbody>
              </table>' ;

    }
    else
    {
        $html .= nl2br(printf($obj, true)) ;
    }
    $html .= '<br /><hr /><br />' ;

    return($html) ;
  }
}
$apr = new Amazon_ProductReport() ;

$apr->DoIt() ;