<?php

@require_once(dirname(__FILE__).'/../../../config/config.inc.php');
@require_once(dirname(__FILE__).'/../../../init.php');

require_once(dirname(__FILE__).'/../classes/amazon.webservice.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.product.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.tools.class.php');

class Amazon_ExportStatistics extends Module
{
  private $_debug = false ;
  private $_europe ;

  private $module   = 'amazon';
  public $name   = 'amazon';

  function __construct()
  {
    global $cookie, $_MODULE, $_LANGADM;

    $langFile = _PS_MODULE_DIR_.$this->module.'/'.Language::getIsoById(intval($cookie->id_lang)).'.php';
    $mySelf   = basename(__FILE__, '.php') ;

    if(file_exists($langFile))
    {
      require_once $langFile;
      foreach($_MODULE  as $key=>$value)
          $_MODULE[str_replace($mySelf . '_' , $this->name . '_', $key)] = $value;
    }
    $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/' ;
    $this->images = $module . 'images/';

    parent::__construct() ;
  }


  function DoIt()
  {
    global $cookie, $cart ;

    $id_lang = intval(Tools::getValue('amazon_lang')) ;

    $callback  = Tools::getValue('callback') ;
    $active  = intval(Tools::getValue('sactive')) ? true : false ;
    $instock = intval(Tools::getValue('sinstock')) ? true : false ;

    $action = strval(Tools::getValue('action')) ;
    $option = strval(Tools::getValue('option')) ;

    if ( intval(Tools::getValue('europe')) )
        $this->europe = 1 ;
    else
        $this->europe = 0 ;

    $this->_debug = unserialize(base64_decode(Configuration::get('AMAZON_DEBUG'))) ;

    // Init
    //
    $amazon = Amazon_Tools::selectPlatforms($id_lang, $this->_debug) ;


    if ( $this->_debug )
        echo nl2br(print_r($amazon['auth'], true) .print_r($amazon['params'], true) . print_r($amazon['platforms'], true));

    if ( ! ($this->_amazonApi = new Service($amazon['auth'], $amazon['params'], $amazon['platforms'], $this->_debug)) )
    {
      echo $this->l('Unable to login') . $cr ;
      die;
    }

    switch($action)
    {
        case 'submission-report' :
            $submissions = unserialize(base64_decode(Configuration::get('AMAZON_LAST_SUBMISSIONS'))) ;

            var_dump($submissions[$id_lang]) ;

            $response = $this->_amazonApi->getFeedSubmissionResult($submissions[$id_lang]['products']) ;

            $pass = $response === false || $response === null ;

            $errors = '' ;
            $messages = '' ;

            if ( isset($response->Message->ProcessingReport) )
            {
                $output = ob_get_clean() ;

                $summary = $response->Message->ProcessingReport->ProcessingSummary ;

#             var_dump($response->Message->ProcessingReport->Result) ;die ;
                if ( isset($response->Message->ProcessingReport->Result) )
                {
                    foreach($response->Message->ProcessingReport->Result as $result)
                    {
                      if ( isset($result->ResultCode) && $result->ResultCode == 'Error' )
                      {
                        $errors .= nl2br(sprintf('Error %d: %s' . "\n",
                          $result->ResultMessageCode,
                          $result->ResultDescription)) ;
                      }
                      else
                      {
                        $errors .= nl2br(sprintf('Message: %s' . "\n",
                          $result->ResultDescription)) ;
                      }
                    }
                    $json  = Amazon_Tools::jsonEncode(
                        array(
                            'MessagesProcessed' => strval($summary->MessagesProcessed),
                            'MessagesSuccessful' => strval($summary->MessagesSuccessful),
                            'MessagesWithError' => strval($summary->MessagesWithError),
                            'MessagesWithWarning' => strval($summary->MessagesWithWarning),
                            'error' => ! $pass,
                            'output' => $output,
                            'errors' => $errors

                        ) ) ;
                }
                else
                {
                    $json  = Amazon_Tools::jsonEncode(
                        array(
                            'error' => true,
                            'output' => $output,
                            'errors' => nl2br(print_r($response,true))
                            ));
                }


            }
            else
            {
                 $output = ob_get_clean() ;

                 $json  = Amazon_Tools::jsonEncode(
                        array(
                            'error' => true,
                            'output' => $output,
                            'errors' => nl2br(print_r($response,true))
                            ));
            }
            die(strval($callback) . '(' . $json . ')');

            break ;

        case 'request-report' :
            // Init
            //
            $this->initAmazon($id_lang) ;

            if ( ($reportRequestId = $this->_amazonApi->requestReport()) )
            {
            	Configuration::updateValue('AMAZON_REPORT_ID', $reportRequestId);
                echo $this->l('Report ID') . ": <b> $reportRequestId </b><br>" ;
            }
            else
            {
                echo $this->l('Unable to generate a report, please again try in few minutes...') ;
            }
            break ;
        case 'display-report' :
            // Init
            //
            $this->initAmazon($id_lang) ;
            $reportRequestId = Configuration::get('AMAZON_REPORT_ID') ;

            $products =  $this->_amazonApi->getProducts($reportRequestId);

            echo $this->l('Report ID') . ": <b> $reportRequestId </b><br>" ;
            echo "<pre>" ;
            print_r($products) ;
            echo "</pre>" ;

            break ;

        case 'exportf' :

          // Build function
          //
          $function = ("get_" . $option) ;

          if ( ! ($content = Amazon_Product::$function($id_lang, $active, $instock)) )
          {
            echo $this->l('Unrecoverable error, the query was failed !') ;
            die;
          }
          $csv = $this->exportcsv($content) ;

          ob_clean() ;

          header('Pragma: public');
          header('Cache-Control: public');
          header('Content-Type: text/xml; charset=utf-8');
          header('Content-Disposition: attachment; filename="'.basename(__FILE__,'php').'.csv"');

          echo $csv ;

          break ;
        case 'statistics' :
          $state = array() ;
          $state['Products']                      = $this->l('Shop Products') ;
          $state['Empty_Reference']               = $this->l('Empty Reference') ;
          $state['Duplicate_References']          = $this->l('Duplicate References') ;
          $state['With_Reference']                = $this->l('With References') ;
          $state['EAN13_And_Reference']           = $this->l('EAN13 and Reference') ;
          $state['EAN13_But_No_Reference']        = $this->l('EAN13 but no Reference') ;
          $state['Reference_But_No_EAN13']        = $this->l('Reference but no EAN13') ;
          $state['Total_Products_Sold_WO_Taxes']  = $this->l('Sum of sold products w/o Taxes') ;


          if ( ! ($statistics = Amazon_Product::getStatistics($id_lang, $active, $instock)) )
          {
            echo $this->l('Unrecoverable error, the query was failed !') ;
            die;
          }

          echo '<table id="statistics-table">' ;
          echo '<thead><tr><td class="description">' . $this->l('Description') . '</td><td>' . $this->l('Value/Qty') . '</td><td>' . $this->l('Percent') . '</td><td class="picto">' . $this->l('Download') . '</td></tr></thead>' ;
          echo '<tbody>' ;

          foreach($statistics as $key => $val)
          {
            if ( preg_match('/Products/', $key) )
              $baseUnit = intval($val) ;
              
            echo '<tr><td>' ;
            echo $state[$key] ;
            echo '</td><td class="valueb">'  ;
            
            if ( is_float($val+1) )
              echo Tools::displayPrice($val) ;
            else
              echo $val ;
              
            if ( $val && ! is_float($val+1) )
            {
              $percent = $baseUnit > $val ? $val / $baseUnit : $baseUnit / $val ;
              $percent *= 100 ;
                  
              echo '</td><td class="value" style="color:' . ($percent > 50 ? 'green' : 'navy') . ';">'  ;
          
              printf('%.02f%%', $percent) ;
              
              echo '</td><td class="picto">' ;
              
              if ( preg_match('/Empty_Reference|Duplicate_References|EAN13_But_No_Reference|Reference_But_No_EAN13/', $key) )
              {
                $download_info = $this->l('Download the list of :') . $state[$key] ;
                echo '<img src="../img/admin/export.gif" class="exportf" alt="' . $download_info . '" rel="'.$key.'" title="' . $download_info . '" />' ;
              }
            }  
            echo "</td>" ;
            echo "</tr>\n"  ;      
              
          }
          echo '</tbody></table>' ;
        break ;
    }
  }
    
  private function exportcsv($contents)
  {
    if(!count($contents))
      return null;


    $fieldheader = array_keys($contents[0]) ;
    echo "<br >";
    $fieldheader = array('id_product', 'reference', 'ean13', 'quantity', 'price', 'supplier_reference', 'date_add', 'date_upd');


    array_unshift($contents, $fieldheader);

    $tmpfile = tempnam(dirname(__FILE__), 'exportcsv');
    $fp = fopen($tmpfile, 'w');
    foreach ($contents as $item) {
      fputcsv($fp, $item, ',', '"');
    }
    fclose($fp);
    $buf = file_get_contents($tmpfile);
    @unlink($tmpfile);
    return $buf;
  }
}
$aes = new Amazon_ExportStatistics() ;

$aes->DoIt() ;