<?php


@require_once(dirname(__FILE__).'/../../../config/config.inc.php');
@require_once(dirname(__FILE__).'/../../../init.php');

require_once(dirname(__FILE__).'/../amazon.php');

require_once(dirname(__FILE__).'/../classes/amazon.product.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.order.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.webservice.class.php');

class Amazon_BulkMode extends Module
{
  private $_auth = array() ;
  private $_param = array() ;
  private $_europe = array() ;
  private $_amazonApi = array() ;
  private $_debug = false ;
  private $_id_lang = 1 ;

  private $module   = 'amazon';
  public $name   = 'amazon';

  private $_amazon ;

  function __construct()
  {
    global $cookie, $_MODULE, $_LANGADM;

    $id_lang = isset($cookie->id_lang) ? $cookie->id_lang : Configuration::get('PS_LANG_DEFAULT') ;
    $langFile = _PS_MODULE_DIR_.$this->module.'/'.Language::getIsoById(intval($id_lang)).'.php';
    $mySelf   = basename(__FILE__, '.php') ;

    if(file_exists($langFile))
    {
      require_once $langFile;
      foreach($_MODULE  as $key=>$value)
          $_MODULE[str_replace($mySelf . '_' , $this->name . '_', $key)] = $value;
    }
    $module = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->module . '/' ;
    $this->images = $module . 'images/';

    $debugMode       = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG'))) ;
    $this->_debug    = intval($debugMode) ? true : false ;

    $this->_amazon = new Amazon() ;

    parent::__construct() ;
  }
  public function l($string, $specific = false, $id_lang = null)
  {
      return( $this->_amazon->l($string, $specific = false, $id_lang = null) ) ;
  }

  
  function BulkUpdate()
  {
    global $cookie, $cart, $argv ;

    $cr = "<br />\n" ; // carriage return


    $tokens = Tools::getValue('cron_token') ;
    $amazon_lang = Tools::getValue('lang') ;
    $europe = intval(Tools::getValue('europe')) ;
    
    
    $id_lang = Language::getIdByIso($amazon_lang) ;
     
    if ( ! Amazon_Tools::checkToken($tokens, $id_lang, $europe) )
        die('Wrong Token') ;

    if ( ! $amazon_lang )
    {
        echo $this->l('No selected language, nothing to do...') ;
        die ;
    }
    $debug = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG')));
    $debug = intval($debug) ? true : false;
    
    // Orders States
    //
    $preparation_state = unserialize(base64_decode(Amazon::configurationGet('AMAZON_PREPARATION_STATE')));
    $sent_state = unserialize(base64_decode(Amazon::configurationGet('AMAZON_SENT_STATE')));      
    $actives = unserialize(base64_decode(Amazon::configurationGet('AMAZON_ACTIVE')));
    
    $amazon_carriers = unserialize(base64_decode(Amazon::configurationGet('AMAZON_CARRIER_DEFAULT')));

    $id_lang_list = '' ;
    
    // Making id_lang_list to select orders
    //
    if ( $europe )
    {
        foreach (Amazon_Tools::languages() as $language)
        {
            if ( ! isset($actives[ $language['id_lang'] ]) || ! $actives[ $language['id_lang'] ]  )
                    continue ;
            $id_lang_list .=  $language['id_lang'] . ',' ;
        }
        $id_lang_list = rtrim($id_lang_list, ',') ;
        
    }
    else
    {
        $id_lang_list = Language::getIdByIso($amazon_lang) ;
        
    }

    $id_lang = Language::getIdByIso($amazon_lang) ;

    // Fetch Orders
    if ( ! ($orders = Amazon_Order::getMarketplaceOrdersStatesByIdLang($id_lang_list, $sent_state)) )
    {
        printf( $this->l('No Orders - exiting normally') . "\n") ;
        exit ;
    } 
 
    $order_list = array() ;
    $carrier_list = array() ;
    $shipping_number_list = array(); 
    
    if ( $debug )
        printf('Orders: %s' . "\n", print_r($orders, true)) ;
    
    foreach($orders as $order)
    {
        $id_lang = $order['id_lang'] ; 
        
        $amazon_carrier = isset($amazon_carriers[$id_lang]) ? $amazon_carriers[$id_lang] : false;
        
        if ( ! $amazon_carrier )
        {
            if ( $debug )
                printf('%s:%d %s %d (%d)' . "\n", basename(__FILE__), __LINE__, $this->l('Skipping order - Empty carrier for order #'), $order['id_order'], $id_lang) ;
            continue ;
        }
        
        // Building array for Amazon API
        //
        $order_list[] = $order['mp_order_id'] ;
        $ps_order_list[] = $order['id_order'] ;
        $carrier_list[] = $amazon_carrier ;
        $shipping_number_list[] = $order['shipping_number'] ;
        
        if ( $debug )
            printf('Order: %s Carrier: %s Shipping Number: %s' . "\n", $order['mp_order_id'], $amazon_carrier, $order['shipping_number']) ;
    }
    
    if ( ! count($order_list) )
    {
        if ( $debug )
            printf( $this->l('No Orders - exiting normally') . "\n") ;
        exit ;        
    }

    // Init Amazon
    //
    $platform = Amazon_Tools::selectPlatform($id_lang, $debug) ;
    
    if ( $debug )
        echo nl2br(print_r($platform['auth'], true) .print_r($platform['params'], true) . print_r($platform['platforms'], true));

    if ( ! ($amazonApi = new Service($platform['auth'], $platform['params'], null, $debug)) )
    {
      echo $this->l('Unable to login') . $cr ;
      die;
    }    

    // Submitting Orders
    //
    if ( ! ($result = $amazonApi->confirmMultipleOrders($order_list, $carrier_list, $shipping_number_list)) )
    {
        printf( $this->l('Unable to send data to Amazon') . "\n") ;        
        die ;
    }       
    
    if ( isset($result->SubmitFeedResult->FeedSubmissionInfo->FeedSubmissionId) )
    {
        foreach($ps_order_list as $id_order)
        {
            Amazon_Order::updateMarketplaceStatus($id_order, Amazon_Order::Checked) ;
        }
        printf('%s %s', $this->l('Data sucessfully submitted, FeedSubmissionId: '), $result->SubmitFeedResult->FeedSubmissionInfo->FeedSubmissionId) ;
    }
    else
    {
        printf('Amazon Returns: %s', print_r($result, true)) ;
    }
    
  }
  
}
$amazonBulkMode = new Amazon_BulkMode ;
$amazonBulkMode->BulkUpdate() ;
?>