<?php


require_once(dirname(__FILE__).'/../../../config/config.inc.php');

require_once(dirname(__FILE__).'/../classes/amazon.tools.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.webservice.class.php');
require_once(dirname(__FILE__).'/../classes/amazon.specificfield.class.php');

require_once(dirname(__FILE__).'/../amazon.php');

require_once(dirname(__FILE__).'/../validate/Node.php') ;
require_once(dirname(__FILE__).'/../validate/XmlDataType.php') ;
require_once(dirname(__FILE__).'/../validate/XmlRestriction.php') ;
require_once(dirname(__FILE__).'/../validate/AmazonValidator.php') ;
require_once(dirname(__FILE__).'/../validate/AmazonXSD.php') ;

class AmazonXSDOperations extends Module
{
    private $module   = 'amazon';
    public  $name   = 'amazon';

    private $_amazon ;
    private $_debug ;
    
    private static $features    = null ;
    
    public $id_lang ;
    
    function __construct()
    {
        global $cookie, $_MODULE;

        $this->id_lang = (int)Tools::getValue('id_lang');
        
        if ( ! $this->id_lang ) 
            $this->id_lang = Configuration::get('PS_LANG_DEFAULT') ;
        
        $langFile = _PS_MODULE_DIR_.$this->name.'/'.Language::getIsoById($this->id_lang).'.php';
        $mySelf   = basename(__FILE__, '.php') ;

        if(file_exists($langFile))
        {
            require_once $langFile;
            foreach($_MODULE  as $key=>$value)
                $_MODULE[str_replace($mySelf . '_' , $this->name . '_', $key)] = $value;
        }

        $this->path = _PS_MODULE_DIR_ . '/' . $this->name . '/' ;
        $this->url = __PS_BASE_URI__ . basename(_PS_MODULE_DIR_) . '/' . $this->name ;
        $this->images = $this->url . '/images/';

        $debugMode       = unserialize(base64_decode(Amazon::configurationGet('AMAZON_DEBUG'))) ;
        $this->_debug    = intval($debugMode) ? true : false ;

        // Load Features Once
        if ( self::$features === null )
        {
            self::$features = Feature::getFeatures($this->id_lang) ;
        }
        
        $this->Amazon = new Amazon() ;

        parent::__construct() ;
    }
    public function l($string, $specific = false, $id_lang = null)
    {
      return( $this->Amazon->l($string, $specific = false, $id_lang = null) ) ;
    }

    public function Dispatch()
    {
       $data = array() ;
       switch( Tools::getValue('action'))
       {
         case 'fetch' :
            $data = $this->Fetch() ;
            break ;
         case 'extrafields' :
            $data = $this->ExtraFields() ;
            break ;
         default :
             die($this->l('No action was choose')) ;
       }
       /* Unused !!!
        
       if ( ! $data )
             die(sprintf('%s/%d: %s', basename(__FILE__), __LINE__, $this->l('The function didn\'t return any data'))) ;       
       
       $callback = Tools::getValue('callback') ;
        
        $json =  Amazon_Tools::jsonEncode( array('data' => $data, 'errors' => $this->errors) ) ;
                
        die( strval($callback) . '(' . $json  . ')' ) ;
        */
    }
 
    /*
     * Specific for Fashion/Clothing
     */
    public function OptionnalFashionFields(&$specificFields, $shoes = false)
    {
        $optionalFashionFields = array('Season', 'OuterMaterial', 'InnerMaterial', 'ModelYear', 'Color') ;

        if ( $shoes )
            $optionalFashionFields[] = 'SoleMaterial' ;
            
        foreach($optionalFashionFields as $field)
        {
            $specificFields['field'][$field] = null ;
            $specificFields['characteristic'][$field] = 0 ;             
        }
    }

    /*
     * Amazon specific fields
     */
    public function ExtraFields()
    {
        $profile = Tools::getValue('profile') ;
        $universe = Tools::getValue('universe') ;
        $productType = Tools::getValue('selected') ;
                
        $productFactory = new AmazonXSD($universe.'.xsd');
        $p = $productFactory->getInstance();

        $required = AmazonXSD::getMandatoryFields($p->ProductData);
        
        // Shoes
        if ( isset($p->ProductData->Shoes->ClassificationData->Department) )
        {
            $specificFields = array() ;
            $specificFields[$profile]['field']['Department'] = null ;
            $specificFields[$profile]['required']['Department'] = true ;
            
            // Optionnal Settings
            //
            $this->OptionnalFashionFields($specificFields[$profile], true) ;
            
            echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ;   
            
        }       
        // Clothes
        elseif ( isset($p->ProductData->Clothing->ClassificationData) )
        {
            $specificFields = array() ;
            $specificFields[$profile]['field']['Department'] = null ;
            $specificFields[$profile]['required']['Department'] = true ;
            
            // Optionnal Settings
            //
            $this->OptionnalFashionFields($specificFields[$profile]) ;
            
            echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ;            
        }
        // Jewelry
        elseif ( isset($p->ProductData->Jewelry) )
        {
            $variationDatas = $p->ProductData->$universe->ProductType->$productType->VariationData->VariationTheme->allowedValues ;

            $specificFields = array() ;
            $specificFields[$profile]['field']['variant'] = 0 ;
            $specificFields[$profile]['variant'] = array() ;

            /*
             * Exploring Variations :
                <xsd:element name="VariationTheme" minOccurs="0">
                <xsd:simpleType>
                        <xsd:restriction base="xsd:string">
                                <xsd:enumeration value="Length"/>
                                <xsd:enumeration value="MetalType"/>
                                <xsd:enumeration value="Length-MetalType"/>
                                <xsd:enumeration value="Color"/>
                                <xsd:enumeration value="StyleName"/>
                        </xsd:restriction>
                </xsd:simpleType>
                </xsd:element>
             */
            foreach($variationDatas as $variationData)
            {
                $variantNames = explode('-', $variationData) ;
                
                /* Split :
                 * <xsd:enumeration value="Length-MetalType"/> => Length and MetalType
                 */
                $specificFields[$profile]['variant'][$variationData] = array() ;
                
                /*
                 * Item has particular attributes :
                 * <xsd:element name="Length" type="LengthDimension" minOccurs="0"/> => LengthDimension
                 * Values => MM, CM, etc...
                 */
                foreach($variantNames as $name)
                {
                    $specificFields[$profile]['variant'][$variationData][$name] = null ;
                    
                    if ( isset($p->ProductData->$universe->ProductType->$productType->VariationData->$name->attr) )
                    {    
                        $tag = $p->ProductData->$universe->ProductType->$productType->VariationData->$name ;
                        
                        if (isset($tag->attr->unitOfMeasure) && isset($tag->attr->unitOfMeasure->allowedValues) && count($tag->attr->unitOfMeasure->allowedValues))
                        {
                            $specificFields[$profile]['attributes'][$variationData][$name]['unitOfMeasure'] = 0 ; 
                            /*
                             * Serialize possible values for the form
                             * MM, CM, etc...
                             */
                            $specificFields[$profile]['values'][$variationData][$name] = base64_encode(serialize($tag->attr->unitOfMeasure->allowedValues)) ;  
                        }                        
                    }
                }
            }
            echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ;  
        }
        // Other Products
        elseif (isset($p->ProductData->$universe->ProductType->$productType) && $p->ProductData->$universe->ProductType->$productType instanceOf stdClass)
        {
            $f = 0;
            foreach ($p->ProductData->$universe->ProductType->$productType as $name => $tag)
            {
                if (AmazonXSD::isMandatoryField($tag) && $tag instanceOf StdClass)
                {
                    if ($tag->type == 'text' || $tag->type == 'number')
                    {
                        $specificFields = array() ;
                        $specificFields[$profile]['field'][$name] = null ;
                        $specificFields[$profile]['required'][$name] = true ;                        
                        
                        if (isset($tag->attr->unitOfMeasure) && isset($tag->attr->unitOfMeasure->allowedValues) && count($tag->attr->unitOfMeasure->allowedValues))
                        {
                            $specificFields[$profile]['attributes'][$name] = array() ; 
                            $specificFields[$profile]['attributes'][$name]['unitOfMeasure'] = 0 ; 
                            $specificFields[$profile]['values'][$name] = base64_encode(serialize($tag->attr->unitOfMeasure->allowedValues)) ;  

                        }

                        echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ; 
                    }
                    $f++;
                }
            }
            
            // Some Cases (eg: Toys - Minimum Age)
            //
            foreach($required as $requirements)
            {
                if ( is_object($requirements) )
                    foreach($requirements as $key => $tag)
                    {
                        if ( $key == 'ProductType' )    continue ;
                        
                        if ( $tag instanceOf StdClass )
                        {
                            if(AmazonXSD::isParentWithChildren($key)):
                                $children = get_object_vars($requirements->$key);
                                foreach($children as $keyName=>$tagValue):
                                    if($tagValue instanceOf StdClass):
                                        
                                        $specificFields = array() ;
                                        $specificFields[$profile]['field'][$keyName] = null ;
                                        $specificFields[$profile]['required'][$keyName] = true ;                        

                                        if (isset($tagValue->attr->unitOfMeasure) && isset($tagValue->attr->unitOfMeasure->allowedValues) && count($tagValue->attr->unitOfMeasure->allowedValues))
                                        {
                                            $specificFields[$profile]['attributes'][$keyName] = array() ; 
                                            $specificFields[$profile]['attributes'][$keyName]['unitOfMeasure'] = 0 ; 
                                            $specificFields[$profile]['values'][$keyName] = base64_encode(serialize($tagValue->attr->unitOfMeasure->allowedValues)) ;  

                                        }
                                        echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ;
                                    endif;
                                endforeach;
                            else:
                                $specificFields = array() ;
                                $specificFields[$profile]['field'][$key] = null ;
                                $specificFields[$profile]['required'][$key] = true ;                            
                                
                                echo Amazon_SpecificField::displayFields($profile, $specificFields[$profile]) ;
                            endif;
                        }
                    }
            }
        }
        die ;
    }
    
    /*
     * Fetch categories
     */
    public function Fetch()
    {
        $this->errors[] = array() ;
        $data = array() ;

        if ( ! ($path = Tools::getValue('path')) ) 
        {
            $this->errors[] = sprintf('%s/%d: %s', basename(__FILE__), __LINE__, $this->l('Missing Parameter:') . ' path') ;
            return(false) ;
        }
        
        if ( ! ($selected = Tools::getValue('selected')) ) 
        {
            $this->errors[] = sprintf('%s/%d: %s', basename(__FILE__), __LINE__, $this->l('Missing Parameter: selected')) ; 
            return(false) ;
        }

        $file = $selected . ".xsd" ;

        if ( ! $file )
        {
            $this->errors[] = sprintf('%s/%d: %s', basename(__FILE__), __LINE__, $this->l('Unable to find the XSD file') . ': '.  $selected) ; 
            return(false) ;
        }
            
        $customElements = array(
                "ProductClothing"=>array("ClothingType"=>"Clothing->ClassificationData->ClothingType")
            );

        //get ProductTypes

        $arr = array($selected); 

        foreach($arr as $key=>$category):               
            $productTypes ="";

            try{
                $productFactory = new AmazonXSD($category.'.xsd');                    
                $productInstance = $productFactory->getInstance();

                $item = new stdClass();
                $item->productType = AmazonXSD::getVariationData($productInstance, "ProductType");              

                if ( $item->productType === false )
                    if ( $category == 'Shoes' )
                        $item->productType = AmazonXSD::getVariationData($productInstance, "ClothingType");              
              
                if(isset($customElements[$category])&&$item):                      
                    foreach($customElements[$category] as $keyName=>$el):
                            $e = explode("->", $el);
                            $obj=$productInstance->ProductData;

                            foreach($e as $tag):
                                $obj = $obj->$tag;
                            endforeach;                        
                            # Debug
                            # echo "productInstance->ProductData->$el";
                            $item->$keyName = $obj;
                    endforeach;
                endif;


                if(!$item):                    
                    $item = AmazonXSD::getVariationData($productInstance, "ProductType");
                endif;

                if ( isset($productInstance->ProductData->Shoes->ClothingType->allowedValues) )
                {
                    foreach($productInstance->ProductData->Shoes->ClothingType->allowedValues as $key => $val)
                    {
                        $productTypes[] = strval($val) ;
                    }                    
                }                    
                elseif ( isset($productInstance->ProductData->Clothing->ClassificationData->ClothingType->allowedValues) )
                {
                    foreach($productInstance->ProductData->Clothing->ClassificationData->ClothingType->allowedValues as $key => $val)
                    {
                        $productTypes[] = strval($val) ;
                    }                    
                }        
                // Added 2012-01-04 O.B.
                elseif ( isset($item->productType->allowedValues) )
                {
                    foreach($item->productType->allowedValues as $key => $val)
                    {
                        $productTypes[] = strval($val) ;
                    }                    
                }
                elseif ( isset($item->productType) && ! empty($item->productType) )
                {
 
                    foreach($item->productType as $key => $val)
                    {
                        if ( ! preg_match('/^[A-Z]/', strval($key)) ) continue ;
                       
                        $productTypes[] = strval($key) ;
                    }
                }
                else
                {
                    // No Types
                        $productTypes = null ;
                }

            }
            catch(Exception $e)
            {
                $this->errors[] = sprintf('XML Error:'.  $e->getMessage()) ;
            }              
        endforeach;
     
        
        if ( count($productTypes) )
        {
            echo '<option value="">' . $this->l('Choose') . '</option>' ;
            
            foreach($productTypes as $productType)
            {
                echo '<option value="' . $productType . '" >' . $productType . '</option>' . "\n" ;
            }
        }
        else
        {
            echo '<option value="">' . $this->l('No Data') . '</option>' ;
        }
        die ;
    }
}
$amazonXSDOperations = new AmazonXSDOperations() ;
$amazonXSDOperations->Dispatch() ;

?>